"""
Python module generated from Java source file com.google.common.collect.Multimaps

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import Function
from com.google.common.base import Predicate
from com.google.common.base import Predicates
from com.google.common.base import Supplier
from com.google.common.collect import *
from com.google.common.collect.Maps import EntryTransformer
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations.concurrent import LazyInit
from com.google.j2objc.annotations import Weak
from com.google.j2objc.annotations import WeakOuter
from java.io import IOException
from java.io import ObjectInputStream
from java.io import ObjectOutputStream
from java.io import Serializable
from java.util import AbstractCollection
from java.util import Collections
from java.util import Comparator
from java.util import Iterator
from java.util import NavigableSet
from java.util import NoSuchElementException
from java.util import SortedSet
from java.util import Spliterator
from java.util.function import BiConsumer
from java.util.function import Consumer
from java.util.stream import Collector
from java.util.stream import Stream
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Multimaps:
    """
    Provides static methods acting on or generating a `Multimap`.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/CollectionUtilitiesExplained#multimaps">`Multimaps`</a>.

    Author(s)
    - Louis Wasserman

    Since
    - 2.0
    """

    @staticmethod
    def toMultimap(keyFunction: "java.util.function.Function"["T", "K"], valueFunction: "java.util.function.Function"["T", "V"], multimapSupplier: "java.util.function.Supplier"["M"]) -> "Collector"["T", Any, "M"]:
        """
        Returns a `Collector` accumulating entries into a `Multimap` generated from the
        specified supplier. The keys and values of the entries are the result of applying the provided
        mapping functions to the input elements, accumulated in the encounter order of the stream.
        
        Example:
        
        ````static final ListMultimap<Character, String> FIRST_LETTER_MULTIMAP =
            Stream.of("banana", "apple", "carrot", "asparagus", "cherry")
                .collect(
                    toMultimap(
                         str -> str.charAt(0),
                         str -> str.substring(1),
                         MultimapBuilder.treeKeys().arrayListValues()::build));
        
        // is equivalent to
        
        static final ListMultimap<Character, String> FIRST_LETTER_MULTIMAP;
        
        static {
            FIRST_LETTER_MULTIMAP = MultimapBuilder.treeKeys().arrayListValues().build();
            FIRST_LETTER_MULTIMAP.put('b', "anana");
            FIRST_LETTER_MULTIMAP.put('a', "pple");
            FIRST_LETTER_MULTIMAP.put('a', "sparagus");
            FIRST_LETTER_MULTIMAP.put('c', "arrot");
            FIRST_LETTER_MULTIMAP.put('c', "herry");`
        }```
        
        To collect to an ImmutableMultimap, use either ImmutableSetMultimap.toImmutableSetMultimap or ImmutableListMultimap.toImmutableListMultimap.

        Since
        - 21.0
        """
        ...


    @staticmethod
    def flatteningToMultimap(keyFunction: "java.util.function.Function"["T", "K"], valueFunction: "java.util.function.Function"["T", "Stream"["V"]], multimapSupplier: "java.util.function.Supplier"["M"]) -> "Collector"["T", Any, "M"]:
        """
        Returns a `Collector` accumulating entries into a `Multimap` generated from the
        specified supplier. Each input element is mapped to a key and a stream of values, each of which
        are put into the resulting `Multimap`, in the encounter order of the stream and the
        encounter order of the streams of values.
        
        Example:
        
        ````static final ListMultimap<Character, Character> FIRST_LETTER_MULTIMAP =
            Stream.of("banana", "apple", "carrot", "asparagus", "cherry")
                .collect(
                    flatteningToMultimap(
                         str -> str.charAt(0),
                         str -> str.substring(1).chars().mapToObj(c -> (char) c),
                         MultimapBuilder.linkedHashKeys().arrayListValues()::build));
        
        // is equivalent to
        
        static final ListMultimap<Character, Character> FIRST_LETTER_MULTIMAP;
        
        static {
            FIRST_LETTER_MULTIMAP = MultimapBuilder.linkedHashKeys().arrayListValues().build();
            FIRST_LETTER_MULTIMAP.putAll('b', Arrays.asList('a', 'n', 'a', 'n', 'a'));
            FIRST_LETTER_MULTIMAP.putAll('a', Arrays.asList('p', 'p', 'l', 'e'));
            FIRST_LETTER_MULTIMAP.putAll('c', Arrays.asList('a', 'r', 'r', 'o', 't'));
            FIRST_LETTER_MULTIMAP.putAll('a', Arrays.asList('s', 'p', 'a', 'r', 'a', 'g', 'u', 's'));
            FIRST_LETTER_MULTIMAP.putAll('c', Arrays.asList('h', 'e', 'r', 'r', 'y'));`
        }```

        Since
        - 21.0
        """
        ...


    @staticmethod
    def newMultimap(map: dict["K", Iterable["V"]], factory: "Supplier"[Iterable["V"]]) -> "Multimap"["K", "V"]:
        """
        Creates a new `Multimap` backed by `map`, whose internal value collections are
        generated by `factory`.
        
        **Warning: do not use** this method when the collections returned by `factory`
        implement either List or `Set`! Use the more specific method .newListMultimap, .newSetMultimap or .newSortedSetMultimap instead, to avoid
        very surprising behavior from Multimap.equals.
        
        The `factory`-generated and `map` classes determine the multimap iteration
        order. They also specify the behavior of the `equals`, `hashCode`, and `toString` methods for the multimap and its returned views. However, the multimap's `get`
        method returns instances of a different class than `factory.get()` does.
        
        The multimap is serializable if `map`, `factory`, the collections generated by
        `factory`, and the multimap contents are all serializable.
        
        The multimap is not threadsafe when any concurrent operations update the multimap, even if
        `map` and the instances generated by `factory` are. Concurrent read operations will
        work correctly. To allow concurrent update operations, wrap the multimap with a call to .synchronizedMultimap.
        
        Call this method only when the simpler methods ArrayListMultimap.create(), HashMultimap.create(), LinkedHashMultimap.create(), LinkedListMultimap.create(), TreeMultimap.create(), and TreeMultimap.create(Comparator, Comparator) won't suffice.
        
        Note: the multimap assumes complete ownership over of `map` and the collections
        returned by `factory`. Those objects should not be manually updated and they should not
        use soft, weak, or phantom references.

        Arguments
        - map: place to store the mapping from each key to its corresponding values
        - factory: supplier of new, empty collections that will each hold all values for a given
            key

        Raises
        - IllegalArgumentException: if `map` is not empty
        """
        ...


    @staticmethod
    def newListMultimap(map: dict["K", Iterable["V"]], factory: "Supplier"[list["V"]]) -> "ListMultimap"["K", "V"]:
        """
        Creates a new `ListMultimap` that uses the provided map and factory. It can generate a
        multimap based on arbitrary Map and List classes.
        
        The `factory`-generated and `map` classes determine the multimap iteration
        order. They also specify the behavior of the `equals`, `hashCode`, and `toString` methods for the multimap and its returned views. The multimap's `get`, `removeAll`, and `replaceValues` methods return `RandomAccess` lists if the factory
        does. However, the multimap's `get` method returns instances of a different class than
        does `factory.get()`.
        
        The multimap is serializable if `map`, `factory`, the lists generated by `factory`, and the multimap contents are all serializable.
        
        The multimap is not threadsafe when any concurrent operations update the multimap, even if
        `map` and the instances generated by `factory` are. Concurrent read operations will
        work correctly. To allow concurrent update operations, wrap the multimap with a call to .synchronizedListMultimap.
        
        Call this method only when the simpler methods ArrayListMultimap.create() and LinkedListMultimap.create() won't suffice.
        
        Note: the multimap assumes complete ownership over of `map` and the lists returned by
        `factory`. Those objects should not be manually updated, they should be empty when
        provided, and they should not use soft, weak, or phantom references.

        Arguments
        - map: place to store the mapping from each key to its corresponding values
        - factory: supplier of new, empty lists that will each hold all values for a given key

        Raises
        - IllegalArgumentException: if `map` is not empty
        """
        ...


    @staticmethod
    def newSetMultimap(map: dict["K", Iterable["V"]], factory: "Supplier"[set["V"]]) -> "SetMultimap"["K", "V"]:
        """
        Creates a new `SetMultimap` that uses the provided map and factory. It can generate a
        multimap based on arbitrary Map and Set classes.
        
        The `factory`-generated and `map` classes determine the multimap iteration
        order. They also specify the behavior of the `equals`, `hashCode`, and `toString` methods for the multimap and its returned views. However, the multimap's `get`
        method returns instances of a different class than `factory.get()` does.
        
        The multimap is serializable if `map`, `factory`, the sets generated by `factory`, and the multimap contents are all serializable.
        
        The multimap is not threadsafe when any concurrent operations update the multimap, even if
        `map` and the instances generated by `factory` are. Concurrent read operations will
        work correctly. To allow concurrent update operations, wrap the multimap with a call to .synchronizedSetMultimap.
        
        Call this method only when the simpler methods HashMultimap.create(), LinkedHashMultimap.create(), TreeMultimap.create(), and TreeMultimap.create(Comparator, Comparator) won't suffice.
        
        Note: the multimap assumes complete ownership over of `map` and the sets returned by
        `factory`. Those objects should not be manually updated and they should not use soft,
        weak, or phantom references.

        Arguments
        - map: place to store the mapping from each key to its corresponding values
        - factory: supplier of new, empty sets that will each hold all values for a given key

        Raises
        - IllegalArgumentException: if `map` is not empty
        """
        ...


    @staticmethod
    def newSortedSetMultimap(map: dict["K", Iterable["V"]], factory: "Supplier"["SortedSet"["V"]]) -> "SortedSetMultimap"["K", "V"]:
        """
        Creates a new `SortedSetMultimap` that uses the provided map and factory. It can generate
        a multimap based on arbitrary Map and SortedSet classes.
        
        The `factory`-generated and `map` classes determine the multimap iteration
        order. They also specify the behavior of the `equals`, `hashCode`, and `toString` methods for the multimap and its returned views. However, the multimap's `get`
        method returns instances of a different class than `factory.get()` does.
        
        The multimap is serializable if `map`, `factory`, the sets generated by `factory`, and the multimap contents are all serializable.
        
        The multimap is not threadsafe when any concurrent operations update the multimap, even if
        `map` and the instances generated by `factory` are. Concurrent read operations will
        work correctly. To allow concurrent update operations, wrap the multimap with a call to .synchronizedSortedSetMultimap.
        
        Call this method only when the simpler methods TreeMultimap.create() and TreeMultimap.create(Comparator, Comparator) won't suffice.
        
        Note: the multimap assumes complete ownership over of `map` and the sets returned by
        `factory`. Those objects should not be manually updated and they should not use soft,
        weak, or phantom references.

        Arguments
        - map: place to store the mapping from each key to its corresponding values
        - factory: supplier of new, empty sorted sets that will each hold all values for a given
            key

        Raises
        - IllegalArgumentException: if `map` is not empty
        """
        ...


    @staticmethod
    def invertFrom(source: "Multimap"["V", "K"], dest: "M") -> "M":
        """
        Copies each key-value mapping in `source` into `dest`, with its key and value
        reversed.
        
        If `source` is an ImmutableMultimap, consider using ImmutableMultimap.inverse instead.

        Arguments
        - source: any multimap
        - dest: the multimap to copy into; usually empty

        Returns
        - `dest`
        """
        ...


    @staticmethod
    def synchronizedMultimap(multimap: "Multimap"["K", "V"]) -> "Multimap"["K", "V"]:
        """
        Returns a synchronized (thread-safe) multimap backed by the specified multimap. In order to
        guarantee serial access, it is critical that **all** access to the backing multimap is
        accomplished through the returned multimap.
        
        It is imperative that the user manually synchronize on the returned multimap when accessing
        any of its collection views:
        
        ````Multimap<K, V> multimap = Multimaps.synchronizedMultimap(
            HashMultimap.<K, V>create());
        ...
        Collection<V> values = multimap.get(key);  // Needn't be in synchronized block
        ...
        synchronized (multimap) {  // Synchronizing on multimap, not values!
          Iterator<V> i = values.iterator(); // Must be in synchronized block
          while (i.hasNext()) {
            foo(i.next());`
        }
        }```
        
        Failure to follow this advice may result in non-deterministic behavior.
        
        Note that the generated multimap's Multimap.removeAll and Multimap.replaceValues methods return collections that aren't synchronized.
        
        The returned multimap will be serializable if the specified multimap is serializable.

        Arguments
        - multimap: the multimap to be wrapped in a synchronized view

        Returns
        - a synchronized view of the specified multimap
        """
        ...


    @staticmethod
    def unmodifiableMultimap(delegate: "Multimap"["K", "V"]) -> "Multimap"["K", "V"]:
        """
        Returns an unmodifiable view of the specified multimap. Query operations on the returned
        multimap "read through" to the specified multimap, and attempts to modify the returned
        multimap, either directly or through the multimap's views, result in an `UnsupportedOperationException`.
        
        The returned multimap will be serializable if the specified multimap is serializable.

        Arguments
        - delegate: the multimap for which an unmodifiable view is to be returned

        Returns
        - an unmodifiable view of the specified multimap
        """
        ...


    @staticmethod
    def unmodifiableMultimap(delegate: "ImmutableMultimap"["K", "V"]) -> "Multimap"["K", "V"]:
        """
        Simply returns its argument.

        Since
        - 10.0

        Deprecated
        - no need to use this
        """
        ...


    @staticmethod
    def synchronizedSetMultimap(multimap: "SetMultimap"["K", "V"]) -> "SetMultimap"["K", "V"]:
        """
        Returns a synchronized (thread-safe) `SetMultimap` backed by the specified multimap.
        
        You must follow the warnings described in .synchronizedMultimap.
        
        The returned multimap will be serializable if the specified multimap is serializable.

        Arguments
        - multimap: the multimap to be wrapped

        Returns
        - a synchronized view of the specified multimap
        """
        ...


    @staticmethod
    def unmodifiableSetMultimap(delegate: "SetMultimap"["K", "V"]) -> "SetMultimap"["K", "V"]:
        """
        Returns an unmodifiable view of the specified `SetMultimap`. Query operations on the
        returned multimap "read through" to the specified multimap, and attempts to modify the returned
        multimap, either directly or through the multimap's views, result in an `UnsupportedOperationException`.
        
        The returned multimap will be serializable if the specified multimap is serializable.

        Arguments
        - delegate: the multimap for which an unmodifiable view is to be returned

        Returns
        - an unmodifiable view of the specified multimap
        """
        ...


    @staticmethod
    def unmodifiableSetMultimap(delegate: "ImmutableSetMultimap"["K", "V"]) -> "SetMultimap"["K", "V"]:
        """
        Simply returns its argument.

        Since
        - 10.0

        Deprecated
        - no need to use this
        """
        ...


    @staticmethod
    def synchronizedSortedSetMultimap(multimap: "SortedSetMultimap"["K", "V"]) -> "SortedSetMultimap"["K", "V"]:
        """
        Returns a synchronized (thread-safe) `SortedSetMultimap` backed by the specified
        multimap.
        
        You must follow the warnings described in .synchronizedMultimap.
        
        The returned multimap will be serializable if the specified multimap is serializable.

        Arguments
        - multimap: the multimap to be wrapped

        Returns
        - a synchronized view of the specified multimap
        """
        ...


    @staticmethod
    def unmodifiableSortedSetMultimap(delegate: "SortedSetMultimap"["K", "V"]) -> "SortedSetMultimap"["K", "V"]:
        """
        Returns an unmodifiable view of the specified `SortedSetMultimap`. Query operations on
        the returned multimap "read through" to the specified multimap, and attempts to modify the
        returned multimap, either directly or through the multimap's views, result in an `UnsupportedOperationException`.
        
        The returned multimap will be serializable if the specified multimap is serializable.

        Arguments
        - delegate: the multimap for which an unmodifiable view is to be returned

        Returns
        - an unmodifiable view of the specified multimap
        """
        ...


    @staticmethod
    def synchronizedListMultimap(multimap: "ListMultimap"["K", "V"]) -> "ListMultimap"["K", "V"]:
        """
        Returns a synchronized (thread-safe) `ListMultimap` backed by the specified multimap.
        
        You must follow the warnings described in .synchronizedMultimap.

        Arguments
        - multimap: the multimap to be wrapped

        Returns
        - a synchronized view of the specified multimap
        """
        ...


    @staticmethod
    def unmodifiableListMultimap(delegate: "ListMultimap"["K", "V"]) -> "ListMultimap"["K", "V"]:
        """
        Returns an unmodifiable view of the specified `ListMultimap`. Query operations on the
        returned multimap "read through" to the specified multimap, and attempts to modify the returned
        multimap, either directly or through the multimap's views, result in an `UnsupportedOperationException`.
        
        The returned multimap will be serializable if the specified multimap is serializable.

        Arguments
        - delegate: the multimap for which an unmodifiable view is to be returned

        Returns
        - an unmodifiable view of the specified multimap
        """
        ...


    @staticmethod
    def unmodifiableListMultimap(delegate: "ImmutableListMultimap"["K", "V"]) -> "ListMultimap"["K", "V"]:
        """
        Simply returns its argument.

        Since
        - 10.0

        Deprecated
        - no need to use this
        """
        ...


    @staticmethod
    def asMap(multimap: "ListMultimap"["K", "V"]) -> dict["K", list["V"]]:
        """
        Returns ListMultimap.asMap multimap.asMap(), with its type corrected from `Map<K,
        Collection<V>>` to `Map<K, List<V>>`.

        Since
        - 15.0
        """
        ...


    @staticmethod
    def asMap(multimap: "SetMultimap"["K", "V"]) -> dict["K", set["V"]]:
        """
        Returns SetMultimap.asMap multimap.asMap(), with its type corrected from `Map<K,
        Collection<V>>` to `Map<K, Set<V>>`.

        Since
        - 15.0
        """
        ...


    @staticmethod
    def asMap(multimap: "SortedSetMultimap"["K", "V"]) -> dict["K", "SortedSet"["V"]]:
        """
        Returns SortedSetMultimap.asMap multimap.asMap(), with its type corrected from `Map<K, Collection<V>>` to `Map<K, SortedSet<V>>`.

        Since
        - 15.0
        """
        ...


    @staticmethod
    def asMap(multimap: "Multimap"["K", "V"]) -> dict["K", Iterable["V"]]:
        """
        Returns Multimap.asMap multimap.asMap(). This is provided for parity with the other
        more strongly-typed `asMap()` implementations.

        Since
        - 15.0
        """
        ...


    @staticmethod
    def forMap(map: dict["K", "V"]) -> "SetMultimap"["K", "V"]:
        """
        Returns a multimap view of the specified map. The multimap is backed by the map, so changes to
        the map are reflected in the multimap, and vice versa. If the map is modified while an
        iteration over one of the multimap's collection views is in progress (except through the
        iterator's own `remove` operation, or through the `setValue` operation on a map
        entry returned by the iterator), the results of the iteration are undefined.
        
        The multimap supports mapping removal, which removes the corresponding mapping from the map.
        It does not support any operations which might add mappings, such as `put`, `putAll` or `replaceValues`.
        
        The returned multimap will be serializable if the specified map is serializable.

        Arguments
        - map: the backing map for the returned multimap view
        """
        ...


    @staticmethod
    def transformValues(fromMultimap: "Multimap"["K", "V1"], function: "Function"["V1", "V2"]) -> "Multimap"["K", "V2"]:
        """
        Returns a view of a multimap where each value is transformed by a function. All other
        properties of the multimap, such as iteration order, are left intact. For example, the code:
        
        ````Multimap<String, Integer> multimap =
            ImmutableSetMultimap.of("a", 2, "b", -3, "b", -3, "a", 4, "c", 6);
        Function<Integer, String> square = new Function<Integer, String>() {
            public String apply(Integer in) {
              return Integer.toString(in * in);`
        };
        Multimap<String, String> transformed =
            Multimaps.transformValues(multimap, square);
          System.out.println(transformed);
        }```
        
        ... prints `{a=[4, 16], b=[9, 9], c=[36]`}.
        
        Changes in the underlying multimap are reflected in this view. Conversely, this view
        supports removal operations, and these are reflected in the underlying multimap.
        
        It's acceptable for the underlying multimap to contain null keys, and even null values
        provided that the function is capable of accepting null input. The transformed multimap might
        contain null values, if the function sometimes gives a null result.
        
        The returned multimap is not thread-safe or serializable, even if the underlying multimap
        is. The `equals` and `hashCode` methods of the returned multimap are meaningless,
        since there is not a definition of `equals` or `hashCode` for general collections,
        and `get()` will return a general `Collection` as opposed to a `List` or a
        `Set`.
        
        The function is applied lazily, invoked when needed. This is necessary for the returned
        multimap to be a view, but it means that the function will be applied many times for bulk
        operations like Multimap.containsValue and `Multimap.toString()`. For this to
        perform well, `function` should be fast. To avoid lazy evaluation when the returned
        multimap doesn't need to be a view, copy the returned multimap into a new multimap of your
        choosing.

        Since
        - 7.0
        """
        ...


    @staticmethod
    def transformValues(fromMultimap: "ListMultimap"["K", "V1"], function: "Function"["V1", "V2"]) -> "ListMultimap"["K", "V2"]:
        """
        Returns a view of a `ListMultimap` where each value is transformed by a function. All
        other properties of the multimap, such as iteration order, are left intact. For example, the
        code:
        
        ````ListMultimap<String, Integer> multimap
             = ImmutableListMultimap.of("a", 4, "a", 16, "b", 9);
        Function<Integer, Double> sqrt =
            new Function<Integer, Double>() {
              public Double apply(Integer in) {
                return Math.sqrt((int) in);`
            };
        ListMultimap<String, Double> transformed = Multimaps.transformValues(map,
            sqrt);
        System.out.println(transformed);
        }```
        
        ... prints `{a=[2.0, 4.0], b=[3.0]`}.
        
        Changes in the underlying multimap are reflected in this view. Conversely, this view
        supports removal operations, and these are reflected in the underlying multimap.
        
        It's acceptable for the underlying multimap to contain null keys, and even null values
        provided that the function is capable of accepting null input. The transformed multimap might
        contain null values, if the function sometimes gives a null result.
        
        The returned multimap is not thread-safe or serializable, even if the underlying multimap
        is.
        
        The function is applied lazily, invoked when needed. This is necessary for the returned
        multimap to be a view, but it means that the function will be applied many times for bulk
        operations like Multimap.containsValue and `Multimap.toString()`. For this to
        perform well, `function` should be fast. To avoid lazy evaluation when the returned
        multimap doesn't need to be a view, copy the returned multimap into a new multimap of your
        choosing.

        Since
        - 7.0
        """
        ...


    @staticmethod
    def transformEntries(fromMap: "Multimap"["K", "V1"], transformer: "EntryTransformer"["K", "V1", "V2"]) -> "Multimap"["K", "V2"]:
        """
        Returns a view of a multimap whose values are derived from the original multimap's entries. In
        contrast to .transformValues, this method's entry-transformation logic may depend on
        the key as well as the value.
        
        All other properties of the transformed multimap, such as iteration order, are left intact.
        For example, the code:
        
        ````SetMultimap<String, Integer> multimap =
            ImmutableSetMultimap.of("a", 1, "a", 4, "b", -6);
        EntryTransformer<String, Integer, String> transformer =
            new EntryTransformer<String, Integer, String>() {
              public String transformEntry(String key, Integer value) {
                 return (value >= 0) ? key : "no" + key;`
            };
        Multimap<String, String> transformed =
            Multimaps.transformEntries(multimap, transformer);
        System.out.println(transformed);
        }```
        
        ... prints `{a=[a, a], b=[nob]`}.
        
        Changes in the underlying multimap are reflected in this view. Conversely, this view
        supports removal operations, and these are reflected in the underlying multimap.
        
        It's acceptable for the underlying multimap to contain null keys and null values provided
        that the transformer is capable of accepting null inputs. The transformed multimap might
        contain null values if the transformer sometimes gives a null result.
        
        The returned multimap is not thread-safe or serializable, even if the underlying multimap
        is. The `equals` and `hashCode` methods of the returned multimap are meaningless,
        since there is not a definition of `equals` or `hashCode` for general collections,
        and `get()` will return a general `Collection` as opposed to a `List` or a
        `Set`.
        
        The transformer is applied lazily, invoked when needed. This is necessary for the returned
        multimap to be a view, but it means that the transformer will be applied many times for bulk
        operations like Multimap.containsValue and Object.toString. For this to perform
        well, `transformer` should be fast. To avoid lazy evaluation when the returned multimap
        doesn't need to be a view, copy the returned multimap into a new multimap of your choosing.
        
        **Warning:** This method assumes that for any instance `k` of `EntryTransformer` key type `K`, `k.equals(k2)` implies that `k2` is also of
        type `K`. Using an `EntryTransformer` key type for which this may not hold, such as
        `ArrayList`, may risk a `ClassCastException` when calling methods on the
        transformed multimap.

        Since
        - 7.0
        """
        ...


    @staticmethod
    def transformEntries(fromMap: "ListMultimap"["K", "V1"], transformer: "EntryTransformer"["K", "V1", "V2"]) -> "ListMultimap"["K", "V2"]:
        """
        Returns a view of a `ListMultimap` whose values are derived from the original multimap's
        entries. In contrast to .transformValues(ListMultimap, Function), this method's
        entry-transformation logic may depend on the key as well as the value.
        
        All other properties of the transformed multimap, such as iteration order, are left intact.
        For example, the code:
        
        ````Multimap<String, Integer> multimap =
            ImmutableMultimap.of("a", 1, "a", 4, "b", 6);
        EntryTransformer<String, Integer, String> transformer =
            new EntryTransformer<String, Integer, String>() {
              public String transformEntry(String key, Integer value) {
                return key + value;`
            };
        Multimap<String, String> transformed =
            Multimaps.transformEntries(multimap, transformer);
        System.out.println(transformed);
        }```
        
        ... prints `{"a"=["a1", "a4"], "b"=["b6"]`}.
        
        Changes in the underlying multimap are reflected in this view. Conversely, this view
        supports removal operations, and these are reflected in the underlying multimap.
        
        It's acceptable for the underlying multimap to contain null keys and null values provided
        that the transformer is capable of accepting null inputs. The transformed multimap might
        contain null values if the transformer sometimes gives a null result.
        
        The returned multimap is not thread-safe or serializable, even if the underlying multimap
        is.
        
        The transformer is applied lazily, invoked when needed. This is necessary for the returned
        multimap to be a view, but it means that the transformer will be applied many times for bulk
        operations like Multimap.containsValue and Object.toString. For this to perform
        well, `transformer` should be fast. To avoid lazy evaluation when the returned multimap
        doesn't need to be a view, copy the returned multimap into a new multimap of your choosing.
        
        **Warning:** This method assumes that for any instance `k` of `EntryTransformer` key type `K`, `k.equals(k2)` implies that `k2` is also of
        type `K`. Using an `EntryTransformer` key type for which this may not hold, such as
        `ArrayList`, may risk a `ClassCastException` when calling methods on the
        transformed multimap.

        Since
        - 7.0
        """
        ...


    @staticmethod
    def index(values: Iterable["V"], keyFunction: "Function"["V", "K"]) -> "ImmutableListMultimap"["K", "V"]:
        """
        Creates an index `ImmutableListMultimap` that contains the results of applying a
        specified function to each item in an `Iterable` of values. Each value will be stored as
        a value in the resulting multimap, yielding a multimap with the same size as the input
        iterable. The key used to store that value in the multimap will be the result of calling the
        function on that value. The resulting multimap is created as an immutable snapshot. In the
        returned multimap, keys appear in the order they are first encountered, and the values
        corresponding to each key appear in the same order as they are encountered.
        
        For example,
        
        ````List<String> badGuys =
            Arrays.asList("Inky", "Blinky", "Pinky", "Pinky", "Clyde");
        Function<String, Integer> stringLengthFunction = ...;
        Multimap<Integer, String> index =
            Multimaps.index(badGuys, stringLengthFunction);
        System.out.println(index);````
        
        prints
        
        ````{4=[Inky], 6=[Blinky], 5=[Pinky, Pinky, Clyde]`
        }```
        
        The returned multimap is serializable if its keys and values are all serializable.

        Arguments
        - values: the values to use when constructing the `ImmutableListMultimap`
        - keyFunction: the function used to produce the key for each value

        Returns
        - `ImmutableListMultimap` mapping the result of evaluating the function `keyFunction` on each value in the input collection to that value

        Raises
        - NullPointerException: if any element of `values` is `null`, or if `keyFunction` produces `null` for any key
        """
        ...


    @staticmethod
    def index(values: Iterator["V"], keyFunction: "Function"["V", "K"]) -> "ImmutableListMultimap"["K", "V"]:
        """
        Creates an index `ImmutableListMultimap` that contains the results of applying a
        specified function to each item in an `Iterator` of values. Each value will be stored as
        a value in the resulting multimap, yielding a multimap with the same size as the input
        iterator. The key used to store that value in the multimap will be the result of calling the
        function on that value. The resulting multimap is created as an immutable snapshot. In the
        returned multimap, keys appear in the order they are first encountered, and the values
        corresponding to each key appear in the same order as they are encountered.
        
        For example,
        
        ````List<String> badGuys =
            Arrays.asList("Inky", "Blinky", "Pinky", "Pinky", "Clyde");
        Function<String, Integer> stringLengthFunction = ...;
        Multimap<Integer, String> index =
            Multimaps.index(badGuys.iterator(), stringLengthFunction);
        System.out.println(index);````
        
        prints
        
        ````{4=[Inky], 6=[Blinky], 5=[Pinky, Pinky, Clyde]`
        }```
        
        The returned multimap is serializable if its keys and values are all serializable.

        Arguments
        - values: the values to use when constructing the `ImmutableListMultimap`
        - keyFunction: the function used to produce the key for each value

        Returns
        - `ImmutableListMultimap` mapping the result of evaluating the function `keyFunction` on each value in the input collection to that value

        Raises
        - NullPointerException: if any element of `values` is `null`, or if `keyFunction` produces `null` for any key

        Since
        - 10.0
        """
        ...


    @staticmethod
    def filterKeys(unfiltered: "Multimap"["K", "V"], keyPredicate: "Predicate"["K"]) -> "Multimap"["K", "V"]:
        """
        Returns a multimap containing the mappings in `unfiltered` whose keys satisfy a
        predicate. The returned multimap is a live view of `unfiltered`; changes to one affect
        the other.
        
        The resulting multimap's views have iterators that don't support `remove()`, but all
        other methods are supported by the multimap and its views. When adding a key that doesn't
        satisfy the predicate, the multimap's `put()`, `putAll()`, and `replaceValues()` methods throw an IllegalArgumentException.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered
        multimap or its views, only mappings whose keys satisfy the filter will be removed from the
        underlying multimap.
        
        The returned multimap isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered multimap's methods, such as `size()`, iterate across every
        key/value mapping in the underlying multimap and determine which satisfy the filter. When a
        live view is *not* needed, it may be faster to copy the filtered multimap and use the
        copy.
        
        **Warning:** `keyPredicate` must be *consistent with equals*, as documented at
        Predicate.apply. Do not provide a predicate such as `Predicates.instanceOf(ArrayList.class)`, which is inconsistent with equals.

        Since
        - 11.0
        """
        ...


    @staticmethod
    def filterKeys(unfiltered: "SetMultimap"["K", "V"], keyPredicate: "Predicate"["K"]) -> "SetMultimap"["K", "V"]:
        """
        Returns a multimap containing the mappings in `unfiltered` whose keys satisfy a
        predicate. The returned multimap is a live view of `unfiltered`; changes to one affect
        the other.
        
        The resulting multimap's views have iterators that don't support `remove()`, but all
        other methods are supported by the multimap and its views. When adding a key that doesn't
        satisfy the predicate, the multimap's `put()`, `putAll()`, and `replaceValues()` methods throw an IllegalArgumentException.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered
        multimap or its views, only mappings whose keys satisfy the filter will be removed from the
        underlying multimap.
        
        The returned multimap isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered multimap's methods, such as `size()`, iterate across every
        key/value mapping in the underlying multimap and determine which satisfy the filter. When a
        live view is *not* needed, it may be faster to copy the filtered multimap and use the
        copy.
        
        **Warning:** `keyPredicate` must be *consistent with equals*, as documented at
        Predicate.apply. Do not provide a predicate such as `Predicates.instanceOf(ArrayList.class)`, which is inconsistent with equals.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def filterKeys(unfiltered: "ListMultimap"["K", "V"], keyPredicate: "Predicate"["K"]) -> "ListMultimap"["K", "V"]:
        """
        Returns a multimap containing the mappings in `unfiltered` whose keys satisfy a
        predicate. The returned multimap is a live view of `unfiltered`; changes to one affect
        the other.
        
        The resulting multimap's views have iterators that don't support `remove()`, but all
        other methods are supported by the multimap and its views. When adding a key that doesn't
        satisfy the predicate, the multimap's `put()`, `putAll()`, and `replaceValues()` methods throw an IllegalArgumentException.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered
        multimap or its views, only mappings whose keys satisfy the filter will be removed from the
        underlying multimap.
        
        The returned multimap isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered multimap's methods, such as `size()`, iterate across every
        key/value mapping in the underlying multimap and determine which satisfy the filter. When a
        live view is *not* needed, it may be faster to copy the filtered multimap and use the
        copy.
        
        **Warning:** `keyPredicate` must be *consistent with equals*, as documented at
        Predicate.apply. Do not provide a predicate such as `Predicates.instanceOf(ArrayList.class)`, which is inconsistent with equals.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def filterValues(unfiltered: "Multimap"["K", "V"], valuePredicate: "Predicate"["V"]) -> "Multimap"["K", "V"]:
        """
        Returns a multimap containing the mappings in `unfiltered` whose values satisfy a
        predicate. The returned multimap is a live view of `unfiltered`; changes to one affect
        the other.
        
        The resulting multimap's views have iterators that don't support `remove()`, but all
        other methods are supported by the multimap and its views. When adding a value that doesn't
        satisfy the predicate, the multimap's `put()`, `putAll()`, and `replaceValues()` methods throw an IllegalArgumentException.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered
        multimap or its views, only mappings whose value satisfy the filter will be removed from the
        underlying multimap.
        
        The returned multimap isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered multimap's methods, such as `size()`, iterate across every
        key/value mapping in the underlying multimap and determine which satisfy the filter. When a
        live view is *not* needed, it may be faster to copy the filtered multimap and use the
        copy.
        
        **Warning:** `valuePredicate` must be *consistent with equals*, as documented
        at Predicate.apply. Do not provide a predicate such as `Predicates.instanceOf(ArrayList.class)`, which is inconsistent with equals.

        Since
        - 11.0
        """
        ...


    @staticmethod
    def filterValues(unfiltered: "SetMultimap"["K", "V"], valuePredicate: "Predicate"["V"]) -> "SetMultimap"["K", "V"]:
        """
        Returns a multimap containing the mappings in `unfiltered` whose values satisfy a
        predicate. The returned multimap is a live view of `unfiltered`; changes to one affect
        the other.
        
        The resulting multimap's views have iterators that don't support `remove()`, but all
        other methods are supported by the multimap and its views. When adding a value that doesn't
        satisfy the predicate, the multimap's `put()`, `putAll()`, and `replaceValues()` methods throw an IllegalArgumentException.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered
        multimap or its views, only mappings whose value satisfy the filter will be removed from the
        underlying multimap.
        
        The returned multimap isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered multimap's methods, such as `size()`, iterate across every
        key/value mapping in the underlying multimap and determine which satisfy the filter. When a
        live view is *not* needed, it may be faster to copy the filtered multimap and use the
        copy.
        
        **Warning:** `valuePredicate` must be *consistent with equals*, as documented
        at Predicate.apply. Do not provide a predicate such as `Predicates.instanceOf(ArrayList.class)`, which is inconsistent with equals.

        Since
        - 14.0
        """
        ...


    @staticmethod
    def filterEntries(unfiltered: "Multimap"["K", "V"], entryPredicate: "Predicate"["Entry"["K", "V"]]) -> "Multimap"["K", "V"]:
        """
        Returns a multimap containing the mappings in `unfiltered` that satisfy a predicate. The
        returned multimap is a live view of `unfiltered`; changes to one affect the other.
        
        The resulting multimap's views have iterators that don't support `remove()`, but all
        other methods are supported by the multimap and its views. When adding a key/value pair that
        doesn't satisfy the predicate, multimap's `put()`, `putAll()`, and `replaceValues()` methods throw an IllegalArgumentException.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered
        multimap or its views, only mappings whose keys satisfy the filter will be removed from the
        underlying multimap.
        
        The returned multimap isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered multimap's methods, such as `size()`, iterate across every
        key/value mapping in the underlying multimap and determine which satisfy the filter. When a
        live view is *not* needed, it may be faster to copy the filtered multimap and use the
        copy.
        
        **Warning:** `entryPredicate` must be *consistent with equals*, as documented
        at Predicate.apply.

        Since
        - 11.0
        """
        ...


    @staticmethod
    def filterEntries(unfiltered: "SetMultimap"["K", "V"], entryPredicate: "Predicate"["Entry"["K", "V"]]) -> "SetMultimap"["K", "V"]:
        """
        Returns a multimap containing the mappings in `unfiltered` that satisfy a predicate. The
        returned multimap is a live view of `unfiltered`; changes to one affect the other.
        
        The resulting multimap's views have iterators that don't support `remove()`, but all
        other methods are supported by the multimap and its views. When adding a key/value pair that
        doesn't satisfy the predicate, multimap's `put()`, `putAll()`, and `replaceValues()` methods throw an IllegalArgumentException.
        
        When methods such as `removeAll()` and `clear()` are called on the filtered
        multimap or its views, only mappings whose keys satisfy the filter will be removed from the
        underlying multimap.
        
        The returned multimap isn't threadsafe or serializable, even if `unfiltered` is.
        
        Many of the filtered multimap's methods, such as `size()`, iterate across every
        key/value mapping in the underlying multimap and determine which satisfy the filter. When a
        live view is *not* needed, it may be faster to copy the filtered multimap and use the
        copy.
        
        **Warning:** `entryPredicate` must be *consistent with equals*, as documented
        at Predicate.apply.

        Since
        - 14.0
        """
        ...



"""
Python module generated from Java source file java.util.EventListener

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from typing import Any, Callable, Iterable, Tuple


class EventListener:
    """
    A tagging interface that all event listener interfaces must extend.

    Since
    - 1.1
    """





"""
Python module generated from Java source file org.bukkit.event.block.BlockDispenseArmorEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Block
from org.bukkit.entity import LivingEntity
from org.bukkit.event.block import *
from org.bukkit.inventory import ItemStack
from org.bukkit.util import Vector
from typing import Any, Callable, Iterable, Tuple


class BlockDispenseArmorEvent(BlockDispenseEvent):
    """
    Called when an equippable item is dispensed from a block and equipped on a
    nearby entity.
    
    If a Block Dispense Armor event is cancelled, the equipment will not be
    equipped on the target entity.
    """

    def __init__(self, block: "Block", dispensed: "ItemStack", target: "LivingEntity"):
        ...


    def getTargetEntity(self) -> "LivingEntity":
        """
        Get the living entity on which the armor was dispensed.

        Returns
        - the target entity
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.ArrayTable

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.base import Objects
from com.google.common.collect import *
from com.google.common.collect.Maps import IteratorBasedAbstractMap
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import DoNotCall
from com.google.errorprone.annotations.concurrent import LazyInit
from com.google.j2objc.annotations import WeakOuter
from java.io import Serializable
from java.lang.reflect import Array
from java.util import Arrays
from java.util import Iterator
from java.util import Spliterator
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ArrayTable(AbstractTable, Serializable):
    """
    Fixed-size Table implementation backed by a two-dimensional array.
    
    **Warning:** `ArrayTable` is rarely the Table implementation you want. First,
    it requires that the complete universe of rows and columns be specified at construction time.
    Second, it is always backed by an array large enough to hold a value for every possible
    combination of row and column keys. (This is rarely optimal unless the table is extremely dense.)
    Finally, every possible combination of row and column keys is always considered to have a value
    associated with it: It is not possible to "remove" a value, only to replace it with `null`,
    which will still appear when iterating over the table's contents in a foreach loop or a call to a
    null-hostile method like ImmutableTable.copyOf. For alternatives, please see <a
    href="https://github.com/google/guava/wiki/NewCollectionTypesExplained#table">the wiki</a>.
    
    The allowed row and column keys must be supplied when the table is created. The table always
    contains a mapping for every row key / column pair. The value corresponding to a given row and
    column is null unless another value is provided.
    
    The table's size is constant: the product of the number of supplied row keys and the number of
    supplied column keys. The `remove` and `clear` methods are not supported by the table
    or its views. The .erase and .eraseAll methods may be used instead.
    
    The ordering of the row and column keys provided when the table is constructed determines the
    iteration ordering across rows and columns in the table's views. None of the view iterators
    support Iterator.remove. If the table is modified after an iterator is created, the
    iterator remains valid.
    
    This class requires less memory than the HashBasedTable and TreeBasedTable
    implementations, except when the table is sparse.
    
    Null row keys or column keys are not permitted.
    
    This class provides methods involving the underlying array structure, where the array indices
    correspond to the position of a row or column in the lists of allowed keys and values. See the
    .at, .set, .toArray, .rowKeyList, and .columnKeyList
    methods for more details.
    
    Note that this implementation is not synchronized. If multiple threads access the same cell of
    an `ArrayTable` concurrently and one of the threads modifies its value, there is no
    guarantee that the new value will be fully visible to the other threads. To guarantee that
    modifications are visible, synchronize access to the table. Unlike other `Table`
    implementations, synchronization is unnecessary between a thread that writes to one cell and a
    thread that reads from another.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/NewCollectionTypesExplained#table">`Table`</a>.

    Author(s)
    - Jared Levy

    Since
    - 10.0
    """

    @staticmethod
    def create(rowKeys: Iterable["R"], columnKeys: Iterable["C"]) -> "ArrayTable"["R", "C", "V"]:
        """
        Creates an `ArrayTable` filled with `null`.

        Arguments
        - rowKeys: row keys that may be stored in the generated table
        - columnKeys: column keys that may be stored in the generated table

        Raises
        - NullPointerException: if any of the provided keys is null
        - IllegalArgumentException: if `rowKeys` or `columnKeys` contains duplicates
            or if exactly one of `rowKeys` or `columnKeys` is empty.
        """
        ...


    @staticmethod
    def create(table: "Table"["R", "C", "V"]) -> "ArrayTable"["R", "C", "V"]:
        """
        Creates an `ArrayTable` with the mappings in the provided table.
        
        If `table` includes a mapping with row key `r` and a separate mapping with
        column key `c`, the returned table contains a mapping with row key `r` and column
        key `c`. If that row key / column key pair in not in `table`, the pair maps to
        `null` in the generated table.
        
        The returned table allows subsequent `put` calls with the row keys in `table.rowKeySet()` and the column keys in `table.columnKeySet()`. Calling .put
        with other keys leads to an `IllegalArgumentException`.
        
        The ordering of `table.rowKeySet()` and `table.columnKeySet()` determines the
        row and column iteration ordering of the returned table.

        Raises
        - NullPointerException: if `table` has a null key
        """
        ...


    def rowKeyList(self) -> "ImmutableList"["R"]:
        """
        Returns, as an immutable list, the row keys provided when the table was constructed, including
        those that are mapped to null values only.
        """
        ...


    def columnKeyList(self) -> "ImmutableList"["C"]:
        """
        Returns, as an immutable list, the column keys provided when the table was constructed,
        including those that are mapped to null values only.
        """
        ...


    def at(self, rowIndex: int, columnIndex: int) -> "V":
        """
        Returns the value corresponding to the specified row and column indices. The same value is
        returned by `get(rowKeyList().get(rowIndex), columnKeyList().get(columnIndex))`, but this
        method runs more quickly.

        Arguments
        - rowIndex: position of the row key in .rowKeyList()
        - columnIndex: position of the row key in .columnKeyList()

        Returns
        - the value with the specified row and column

        Raises
        - IndexOutOfBoundsException: if either index is negative, `rowIndex` is greater than
            or equal to the number of allowed row keys, or `columnIndex` is greater than or equal
            to the number of allowed column keys
        """
        ...


    def set(self, rowIndex: int, columnIndex: int, value: "V") -> "V":
        """
        Associates `value` with the specified row and column indices. The logic `put(rowKeyList().get(rowIndex), columnKeyList().get(columnIndex), value)` has the same
        behavior, but this method runs more quickly.

        Arguments
        - rowIndex: position of the row key in .rowKeyList()
        - columnIndex: position of the row key in .columnKeyList()
        - value: value to store in the table

        Returns
        - the previous value with the specified row and column

        Raises
        - IndexOutOfBoundsException: if either index is negative, `rowIndex` is greater than
            or equal to the number of allowed row keys, or `columnIndex` is greater than or equal
            to the number of allowed column keys
        """
        ...


    def toArray(self, valueClass: type["V"]) -> list[list["V"]]:
        """
        Returns a two-dimensional array with the table contents. The row and column indices correspond
        to the positions of the row and column in the iterables provided during table construction. If
        the table lacks a mapping for a given row and column, the corresponding array element is null.
        
        Subsequent table changes will not modify the array, and vice versa.

        Arguments
        - valueClass: class of values stored in the returned array
        """
        ...


    def clear(self) -> None:
        """
        Not supported. Use .eraseAll instead.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Use .eraseAll
        """
        ...


    def eraseAll(self) -> None:
        """
        Associates the value `null` with every pair of allowed row and column keys.
        """
        ...


    def contains(self, rowKey: "Object", columnKey: "Object") -> bool:
        """
        Returns `True` if the provided keys are among the keys provided when the table was
        constructed.
        """
        ...


    def containsColumn(self, columnKey: "Object") -> bool:
        """
        Returns `True` if the provided column key is among the column keys provided when the
        table was constructed.
        """
        ...


    def containsRow(self, rowKey: "Object") -> bool:
        """
        Returns `True` if the provided row key is among the row keys provided when the table was
        constructed.
        """
        ...


    def containsValue(self, value: "Object") -> bool:
        ...


    def get(self, rowKey: "Object", columnKey: "Object") -> "V":
        ...


    def isEmpty(self) -> bool:
        """
        Returns `True` if `rowKeyList().size == 0` or `columnKeyList().size() == 0`.
        """
        ...


    def put(self, rowKey: "R", columnKey: "C", value: "V") -> "V":
        """
        Raises
        - IllegalArgumentException: if `rowKey` is not in .rowKeySet() or `columnKey` is not in .columnKeySet().
        """
        ...


    def putAll(self, table: "Table"["R", "C", "V"]) -> None:
        """
        
        
        If `table` is an `ArrayTable`, its null values will be stored in this table,
        possibly replacing values that were previously non-null.

        Raises
        - NullPointerException: if `table` has a null key
        - IllegalArgumentException: if any of the provided table's row keys or column keys is not
            in .rowKeySet() or .columnKeySet()
        """
        ...


    def remove(self, rowKey: "Object", columnKey: "Object") -> "V":
        """
        Not supported. Use .erase instead.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Use .erase
        """
        ...


    def erase(self, rowKey: "Object", columnKey: "Object") -> "V":
        """
        Associates the value `null` with the specified keys, assuming both keys are valid. If
        either key is null or isn't among the keys provided during construction, this method has no
        effect.
        
        This method is equivalent to `put(rowKey, columnKey, null)` when both provided keys
        are valid.

        Arguments
        - rowKey: row key of mapping to be erased
        - columnKey: column key of mapping to be erased

        Returns
        - the value previously associated with the keys, or `null` if no mapping existed
            for the keys
        """
        ...


    def size(self) -> int:
        ...


    def cellSet(self) -> set["Cell"["R", "C", "V"]]:
        """
        Returns an unmodifiable set of all row key / column key / value triplets. Changes to the table
        will update the returned set.
        
        The returned set's iterator traverses the mappings with the first row key, the mappings with
        the second row key, and so on.
        
        The value in the returned cells may change if the table subsequently changes.

        Returns
        - set of table cells consisting of row key / column key / value triplets
        """
        ...


    def column(self, columnKey: "C") -> dict["R", "V"]:
        """
        Returns a view of all mappings that have the given column key. If the column key isn't in
        .columnKeySet(), an empty immutable map is returned.
        
        Otherwise, for each row key in .rowKeySet(), the returned map associates the row key
        with the corresponding value in the table. Changes to the returned map will update the
        underlying table, and vice versa.

        Arguments
        - columnKey: key of column to search for in the table

        Returns
        - the corresponding map from row keys to values
        """
        ...


    def columnKeySet(self) -> "ImmutableSet"["C"]:
        """
        Returns an immutable set of the valid column keys, including those that are associated with
        null values only.

        Returns
        - immutable set of column keys
        """
        ...


    def columnMap(self) -> dict["C", dict["R", "V"]]:
        ...


    def row(self, rowKey: "R") -> dict["C", "V"]:
        """
        Returns a view of all mappings that have the given row key. If the row key isn't in .rowKeySet(), an empty immutable map is returned.
        
        Otherwise, for each column key in .columnKeySet(), the returned map associates the
        column key with the corresponding value in the table. Changes to the returned map will update
        the underlying table, and vice versa.

        Arguments
        - rowKey: key of row to search for in the table

        Returns
        - the corresponding map from column keys to values
        """
        ...


    def rowKeySet(self) -> "ImmutableSet"["R"]:
        """
        Returns an immutable set of the valid row keys, including those that are associated with null
        values only.

        Returns
        - immutable set of row keys
        """
        ...


    def rowMap(self) -> dict["R", dict["C", "V"]]:
        ...


    def values(self) -> Iterable["V"]:
        """
        Returns an unmodifiable collection of all values, which may contain duplicates. Changes to the
        table will update the returned collection.
        
        The returned collection's iterator traverses the values of the first row key, the values of
        the second row key, and so on.

        Returns
        - collection of values
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.Llama

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.entity import *
from org.bukkit.inventory import LlamaInventory
from typing import Any, Callable, Iterable, Tuple


class Llama(ChestedHorse):
    """
    Represents a Llama.
    """

    def getColor(self) -> "Color":
        """
        Gets the llama's color.

        Returns
        - a Color representing the llama's color
        """
        ...


    def setColor(self, color: "Color") -> None:
        """
        Sets the llama's color.

        Arguments
        - color: a Color for this llama
        """
        ...


    def getStrength(self) -> int:
        """
        Gets the llama's strength. A higher strength llama will have more
        inventory slots and be more threatening to entities.

        Returns
        - llama strength [1,5]
        """
        ...


    def setStrength(self, strength: int) -> None:
        """
        Sets the llama's strength. A higher strength llama will have more
        inventory slots and be more threatening to entities. Inventory slots are
        equal to strength * 3.

        Arguments
        - strength: llama strength [1,5]
        """
        ...


    def getInventory(self) -> "LlamaInventory":
        ...


    class Color(Enum):
        """
        Represents the base color that the llama has.
        """

        CREAMY = 0
        """
        A cream-colored llama.
        """
        WHITE = 1
        """
        A white llama.
        """
        BROWN = 2
        """
        A brown llama.
        """
        GRAY = 3
        """
        A gray llama.
        """



"""
Python module generated from Java source file com.google.common.io.FileWriteMode

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.io import *
from enum import Enum
from typing import Any, Callable, Iterable, Tuple


class FileWriteMode(Enum):
    """
    Modes for opening a file for writing. The default when mode when none is specified is to truncate
    the file before writing.

    Author(s)
    - Colin Decker
    """

    APPEND = 0
    """
    Specifies that writes to the opened file should append to the end of the file.
    """



"""
Python module generated from Java source file org.bukkit.entity.Wolf

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import DyeColor
from org.bukkit import Keyed
from org.bukkit import NamespacedKey
from org.bukkit import Registry
from org.bukkit.entity import *
from org.bukkit.registry import RegistryAware
from typing import Any, Callable, Iterable, Tuple


class Wolf(Tameable, Sittable):
    """
    Represents a Wolf
    """

    def isAngry(self) -> bool:
        """
        Checks if this wolf is angry

        Returns
        - Anger True if angry
        """
        ...


    def setAngry(self, angry: bool) -> None:
        """
        Sets the anger of this wolf.
        
        An angry wolf can not be fed or tamed.

        Arguments
        - angry: True if angry

        See
        - .setTarget(org.bukkit.entity.LivingEntity)
        """
        ...


    def getCollarColor(self) -> "DyeColor":
        """
        Get the collar color of this wolf

        Returns
        - the color of the collar
        """
        ...


    def setCollarColor(self, color: "DyeColor") -> None:
        """
        Set the collar color of this wolf

        Arguments
        - color: the color to apply
        """
        ...


    def isWet(self) -> bool:
        """
        Gets whether the wolf is wet

        Returns
        - Whether the wolf is wet
        """
        ...


    def getTailAngle(self) -> float:
        """
        Gets the wolf's tail angle in radians

        Returns
        - The angle of the wolf's tail in radians
        """
        ...


    def isInterested(self) -> bool:
        """
        Gets if the wolf is interested

        Returns
        - Whether the wolf is interested
        """
        ...


    def setInterested(self, interested: bool) -> None:
        """
        Set wolf to be interested

        Arguments
        - interested: Whether the wolf is interested
        """
        ...


    def getVariant(self) -> "Variant":
        """
        Get the variant of this wolf.

        Returns
        - wolf variant
        """
        ...


    def setVariant(self, variant: "Variant") -> None:
        """
        Set the variant of this wolf.

        Arguments
        - variant: wolf variant
        """
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.Internal

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.util.concurrent import *
from java.time import Duration
from typing import Any, Callable, Iterable, Tuple


class Internal:
    """
    This class is for `com.google.common.util.concurrent` use only!
    """





"""
Python module generated from Java source file org.bukkit.block.Container

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Nameable
from org.bukkit.block import *
from org.bukkit.inventory import BlockInventoryHolder
from org.bukkit.inventory import Inventory
from typing import Any, Callable, Iterable, Tuple


class Container(TileState, BlockInventoryHolder, Lockable, Nameable):
    """
    Represents a captured state of a container block.
    """

    def getInventory(self) -> "Inventory":
        """
        Gets the inventory of the block represented by this block state.
        
        If the block was changed to a different type in the meantime, the
        returned inventory might no longer be valid.
        
        If this block state is not placed this will return the captured inventory
        snapshot instead.

        Returns
        - the inventory
        """
        ...


    def getSnapshotInventory(self) -> "Inventory":
        """
        Gets the captured inventory snapshot of this container.
        
        The returned inventory is not linked to any block. Any modifications to
        the returned inventory will not be applied to the block represented by
        this block state up until .update(boolean, boolean) has been
        called.

        Returns
        - the captured inventory snapshot
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.ImmutableBiMap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import DoNotCall
from java.io import InvalidObjectException
from java.io import ObjectInputStream
from java.util import Arrays
from java.util import Comparator
from java.util.function import BinaryOperator
from java.util.function import Function
from java.util.stream import Collector
from java.util.stream import Collectors
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ImmutableBiMap(ImmutableMap, BiMap):
    """
    A BiMap whose contents will never change, with many other important properties detailed
    at ImmutableCollection.

    Author(s)
    - Jared Levy

    Since
    - 2.0
    """

    @staticmethod
    def toImmutableBiMap(keyFunction: "Function"["T", "K"], valueFunction: "Function"["T", "V"]) -> "Collector"["T", Any, "ImmutableBiMap"["K", "V"]]:
        """
        Returns a Collector that accumulates elements into an `ImmutableBiMap` whose keys
        and values are the result of applying the provided mapping functions to the input elements.
        Entries appear in the result `ImmutableBiMap` in encounter order.
        
        If the mapped keys or values contain duplicates (according to Object.equals(Object)), an `IllegalArgumentException` is thrown when the collection
        operation is performed. (This differs from the `Collector` returned by Collectors.toMap(Function, Function), which throws an `IllegalStateException`.)

        Since
        - 21.0
        """
        ...


    @staticmethod
    def of() -> "ImmutableBiMap"["K", "V"]:
        ...


    @staticmethod
    def of(k1: "K", v1: "V") -> "ImmutableBiMap"["K", "V"]:
        """
        Returns an immutable bimap containing a single entry.
        """
        ...


    @staticmethod
    def of(k1: "K", v1: "V", k2: "K", v2: "V") -> "ImmutableBiMap"["K", "V"]:
        """
        Returns an immutable map containing the given entries, in order.

        Raises
        - IllegalArgumentException: if duplicate keys or values are added
        """
        ...


    @staticmethod
    def of(k1: "K", v1: "V", k2: "K", v2: "V", k3: "K", v3: "V") -> "ImmutableBiMap"["K", "V"]:
        """
        Returns an immutable map containing the given entries, in order.

        Raises
        - IllegalArgumentException: if duplicate keys or values are added
        """
        ...


    @staticmethod
    def of(k1: "K", v1: "V", k2: "K", v2: "V", k3: "K", v3: "V", k4: "K", v4: "V") -> "ImmutableBiMap"["K", "V"]:
        """
        Returns an immutable map containing the given entries, in order.

        Raises
        - IllegalArgumentException: if duplicate keys or values are added
        """
        ...


    @staticmethod
    def of(k1: "K", v1: "V", k2: "K", v2: "V", k3: "K", v3: "V", k4: "K", v4: "V", k5: "K", v5: "V") -> "ImmutableBiMap"["K", "V"]:
        """
        Returns an immutable map containing the given entries, in order.

        Raises
        - IllegalArgumentException: if duplicate keys or values are added
        """
        ...


    @staticmethod
    def of(k1: "K", v1: "V", k2: "K", v2: "V", k3: "K", v3: "V", k4: "K", v4: "V", k5: "K", v5: "V", k6: "K", v6: "V") -> "ImmutableBiMap"["K", "V"]:
        """
        Returns an immutable map containing the given entries, in order.

        Raises
        - IllegalArgumentException: if duplicate keys or values are added

        Since
        - 31.0
        """
        ...


    @staticmethod
    def of(k1: "K", v1: "V", k2: "K", v2: "V", k3: "K", v3: "V", k4: "K", v4: "V", k5: "K", v5: "V", k6: "K", v6: "V", k7: "K", v7: "V") -> "ImmutableBiMap"["K", "V"]:
        """
        Returns an immutable map containing the given entries, in order.

        Raises
        - IllegalArgumentException: if duplicate keys or values are added

        Since
        - 31.0
        """
        ...


    @staticmethod
    def of(k1: "K", v1: "V", k2: "K", v2: "V", k3: "K", v3: "V", k4: "K", v4: "V", k5: "K", v5: "V", k6: "K", v6: "V", k7: "K", v7: "V", k8: "K", v8: "V") -> "ImmutableBiMap"["K", "V"]:
        """
        Returns an immutable map containing the given entries, in order.

        Raises
        - IllegalArgumentException: if duplicate keys or values are added

        Since
        - 31.0
        """
        ...


    @staticmethod
    def of(k1: "K", v1: "V", k2: "K", v2: "V", k3: "K", v3: "V", k4: "K", v4: "V", k5: "K", v5: "V", k6: "K", v6: "V", k7: "K", v7: "V", k8: "K", v8: "V", k9: "K", v9: "V") -> "ImmutableBiMap"["K", "V"]:
        """
        Returns an immutable map containing the given entries, in order.

        Raises
        - IllegalArgumentException: if duplicate keys or values are added

        Since
        - 31.0
        """
        ...


    @staticmethod
    def of(k1: "K", v1: "V", k2: "K", v2: "V", k3: "K", v3: "V", k4: "K", v4: "V", k5: "K", v5: "V", k6: "K", v6: "V", k7: "K", v7: "V", k8: "K", v8: "V", k9: "K", v9: "V", k10: "K", v10: "V") -> "ImmutableBiMap"["K", "V"]:
        """
        Returns an immutable map containing the given entries, in order.

        Raises
        - IllegalArgumentException: if duplicate keys or values are added

        Since
        - 31.0
        """
        ...


    @staticmethod
    def ofEntries(*entries: Tuple["Entry"["K", "V"], ...]) -> "ImmutableBiMap"["K", "V"]:
        """
        Returns an immutable map containing the given entries, in order.

        Raises
        - IllegalArgumentException: if duplicate keys or values are provided

        Since
        - 31.0
        """
        ...


    @staticmethod
    def builder() -> "Builder"["K", "V"]:
        """
        Returns a new builder. The generated builder is equivalent to the builder created by the Builder constructor.
        """
        ...


    @staticmethod
    def builderWithExpectedSize(expectedSize: int) -> "Builder"["K", "V"]:
        """
        Returns a new builder, expecting the specified number of entries to be added.
        
        If `expectedSize` is exactly the number of entries added to the builder before Builder.build is called, the builder is likely to perform better than an unsized .builder() would have.
        
        It is not specified if any performance benefits apply if `expectedSize` is close to,
        but not exactly, the number of entries added to the builder.

        Since
        - 23.1
        """
        ...


    @staticmethod
    def copyOf(map: dict["K", "V"]) -> "ImmutableBiMap"["K", "V"]:
        """
        Returns an immutable bimap containing the same entries as `map`. If `map` somehow
        contains entries with duplicate keys (for example, if it is a `SortedMap` whose
        comparator is not *consistent with equals*), the results of this method are undefined.
        
        The returned `BiMap` iterates over entries in the same order as the `entrySet`
        of the original map.
        
        Despite the method name, this method attempts to avoid actually copying the data when it is
        safe to do so. The exact circumstances under which a copy will or will not be performed are
        undocumented and subject to change.

        Raises
        - IllegalArgumentException: if two keys have the same value or two values have the same
            key
        - NullPointerException: if any key or value in `map` is null
        """
        ...


    @staticmethod
    def copyOf(entries: Iterable["Entry"["K", "V"]]) -> "ImmutableBiMap"["K", "V"]:
        """
        Returns an immutable bimap containing the given entries. The returned bimap iterates over
        entries in the same order as the original iterable.

        Raises
        - IllegalArgumentException: if two keys have the same value or two values have the same
            key
        - NullPointerException: if any key, value, or entry is null

        Since
        - 19.0
        """
        ...


    def inverse(self) -> "ImmutableBiMap"["V", "K"]:
        """
        
        
        The inverse of an `ImmutableBiMap` is another `ImmutableBiMap`.
        """
        ...


    def values(self) -> "ImmutableSet"["V"]:
        """
        Returns an immutable set of the values in this map, in the same order they appear in .entrySet.
        """
        ...


    def forcePut(self, key: "K", value: "V") -> "V":
        """
        Guaranteed to throw an exception and leave the bimap unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    @staticmethod
    def toImmutableMap(keyFunction: "Function"["T", "K"], valueFunction: "Function"["T", "V"]) -> "Collector"["T", Any, "ImmutableMap"["K", "V"]]:
        """
        Not supported. Use .toImmutableBiMap instead. This method exists only to hide ImmutableMap.toImmutableMap(Function, Function) from consumers of `ImmutableBiMap`.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Use ImmutableBiMap.toImmutableBiMap.
        """
        ...


    @staticmethod
    def toImmutableMap(keyFunction: "Function"["T", "K"], valueFunction: "Function"["T", "V"], mergeFunction: "BinaryOperator"["V"]) -> "Collector"["T", Any, "ImmutableMap"["K", "V"]]:
        """
        Not supported. This method does not make sense for `BiMap`. This method exists only to
        hide ImmutableMap.toImmutableMap(Function, Function, BinaryOperator) from consumers of
        `ImmutableBiMap`.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - 
        """
        ...


    class Builder(Builder):
        """
        A builder for creating immutable bimap instances, especially `public static final` bimaps
        ("constant bimaps"). Example:
        
        ````static final ImmutableBiMap<String, Integer> WORD_TO_INT =
            new ImmutableBiMap.Builder<String, Integer>()
                .put("one", 1)
                .put("two", 2)
                .put("three", 3)
                .buildOrThrow();````
        
        For *small* immutable bimaps, the `ImmutableBiMap.of()` methods are even more
        convenient.
        
        By default, a `Builder` will generate bimaps that iterate over entries in the order
        they were inserted into the builder. For example, in the above example, `WORD_TO_INT.entrySet()` is guaranteed to iterate over the entries in the order `"one"=1,
        "two"=2, "three"=3`, and `keySet()` and `values()` respect the same order. If you
        want a different order, consider using .orderEntriesByValue(Comparator), which changes
        this builder to sort entries by value.
        
        Builder instances can be reused - it is safe to call .buildOrThrow multiple times to
        build multiple bimaps in series. Each bimap is a superset of the bimaps created before it.

        Since
        - 2.0
        """

        def __init__(self):
            """
            Creates a new builder. The returned builder is equivalent to the builder generated by ImmutableBiMap.builder.
            """
            ...


        def put(self, key: "K", value: "V") -> "Builder"["K", "V"]:
            """
            Associates `key` with `value` in the built bimap. Duplicate keys or values are
            not allowed, and will cause .build to fail.
            """
            ...


        def put(self, entry: "Entry"["K", "V"]) -> "Builder"["K", "V"]:
            """
            Adds the given `entry` to the bimap. Duplicate keys or values are not allowed, and will
            cause .build to fail.

            Since
            - 19.0
            """
            ...


        def putAll(self, map: dict["K", "V"]) -> "Builder"["K", "V"]:
            """
            Associates all of the given map's keys and values in the built bimap. Duplicate keys or
            values are not allowed, and will cause .build to fail.

            Raises
            - NullPointerException: if any key or value in `map` is null
            """
            ...


        def putAll(self, entries: Iterable["Entry"["K", "V"]]) -> "Builder"["K", "V"]:
            """
            Adds all of the given entries to the built bimap. Duplicate keys or values are not allowed,
            and will cause .build to fail.

            Raises
            - NullPointerException: if any key, value, or entry is null

            Since
            - 19.0
            """
            ...


        def orderEntriesByValue(self, valueComparator: "Comparator"["V"]) -> "Builder"["K", "V"]:
            """
            Configures this `Builder` to order entries by value according to the specified
            comparator.
            
            The sort order is stable, that is, if two entries have values that compare as equivalent,
            the entry that was inserted first will be first in the built map's iteration order.

            Raises
            - IllegalStateException: if this method was already called

            Since
            - 19.0
            """
            ...


        def build(self) -> "ImmutableBiMap"["K", "V"]:
            """
            Returns a newly-created immutable bimap. The iteration order of the returned bimap is the
            order in which entries were inserted into the builder, unless .orderEntriesByValue
            was called, in which case entries are sorted by value.
            
            Prefer the equivalent method .buildOrThrow() to make it explicit that the method
            will throw an exception if there are duplicate keys or values. The `build()` method
            will soon be deprecated.

            Raises
            - IllegalArgumentException: if duplicate keys or values were added
            """
            ...


        def buildOrThrow(self) -> "ImmutableBiMap"["K", "V"]:
            """
            Returns a newly-created immutable bimap, or throws an exception if any key or value was added
            more than once. The iteration order of the returned bimap is the order in which entries were
            inserted into the builder, unless .orderEntriesByValue was called, in which case
            entries are sorted by value.

            Raises
            - IllegalArgumentException: if duplicate keys or values were added

            Since
            - 31.0
            """
            ...


        def buildKeepingLast(self) -> "ImmutableBiMap"["K", "V"]:
            """
            Throws UnsupportedOperationException. This method is inherited from ImmutableMap.Builder, but it does not make sense for bimaps.

            Raises
            - UnsupportedOperationException: always

            Since
            - 31.1

            Deprecated
            - This method does not make sense for bimaps and should not be called.
            """
            ...



"""
Python module generated from Java source file org.bukkit.event.server.ServiceRegisterEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.event import HandlerList
from org.bukkit.event.server import *
from org.bukkit.plugin import RegisteredServiceProvider
from typing import Any, Callable, Iterable, Tuple


class ServiceRegisterEvent(ServiceEvent):
    """
    This event is called when a service is registered.
    
    Warning: The order in which register and unregister events are called
    should not be relied upon.
    """

    def __init__(self, registeredProvider: "RegisteredServiceProvider"[Any]):
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerFishEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.entity import Entity
from org.bukkit.entity import FishHook
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from org.bukkit.inventory import EquipmentSlot
from typing import Any, Callable, Iterable, Tuple


class PlayerFishEvent(PlayerEvent, Cancellable):
    """
    Thrown when a player is fishing
    """

    def __init__(self, player: "Player", entity: "Entity", hookEntity: "FishHook", hand: "EquipmentSlot", state: "State"):
        ...


    def __init__(self, player: "Player", entity: "Entity", hookEntity: "FishHook", state: "State"):
        ...


    def getCaught(self) -> "Entity":
        """
        Gets the entity caught by the player.
        
        If player has fished successfully, the result may be cast to org.bukkit.entity.Item.

        Returns
        - Entity caught by the player, Entity if fishing, and null if
            bobber has gotten stuck in the ground or nothing has been caught
        """
        ...


    def getHook(self) -> "FishHook":
        """
        Gets the fishing hook.

        Returns
        - the entity representing the fishing hook/bobber.
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getExpToDrop(self) -> int:
        """
        Gets the amount of experience received when fishing.
        
        Note: This value has no default effect unless the event state is State.CAUGHT_FISH.

        Returns
        - the amount of experience to drop
        """
        ...


    def setExpToDrop(self, amount: int) -> None:
        """
        Sets the amount of experience received when fishing.
        
        Note: This value has no default effect unless the event state is State.CAUGHT_FISH.

        Arguments
        - amount: the amount of experience to drop
        """
        ...


    def getHand(self) -> "EquipmentSlot":
        """
        Get the hand that was used in this event.
        
        The hand used is only present when the event state is State.FISHING.
        In all other states, the hand is null.

        Returns
        - the hand
        """
        ...


    def getState(self) -> "State":
        """
        Gets the state of the fishing

        Returns
        - A State detailing the state of the fishing
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...


    class State(Enum):
        """
        An enum to specify the state of the fishing
        """

        FISHING = 0
        """
        When a player is fishing, ie casting the line out.
        """
        CAUGHT_FISH = 1
        """
        When a player has successfully caught a fish and is reeling it in. In
        this instance, a "fish" is any item retrieved from water as a result
        of fishing, ie an item, but not necessarily a fish.
        """
        CAUGHT_ENTITY = 2
        """
        When a player has successfully caught an entity. This refers to any
        already spawned entity in the world that has been hooked directly by
        the rod.
        """
        IN_GROUND = 3
        """
        When a bobber is stuck in the ground.
        """
        FAILED_ATTEMPT = 4
        """
        When a player fails to catch a bite while fishing usually due to
        poor timing.
        """
        REEL_IN = 5
        """
        When a player reels in their hook without receiving any bites.
        """
        BITE = 6
        """
        Called when there is a bite on the hook and it is ready to be reeled
        in.
        """



from .PoweredMinecart import PoweredMinecart
from .StorageMinecart import StorageMinecart
from .HopperMinecart import HopperMinecart
from .SpawnerMinecart import SpawnerMinecart
from .ExplosiveMinecart import ExplosiveMinecart
from .CommandMinecart import CommandMinecart
from .RideableMinecart import RideableMinecart


"""
Python module generated from Java source file java.util.jar.Manifest

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import DataOutputStream
from java.io import FilterInputStream
from java.io import IOException
from java.io import InputStream
from java.io import OutputStream
from java.util.jar import *
from sun.nio.cs import UTF_8
from sun.security.util import SecurityProperties
from typing import Any, Callable, Iterable, Tuple


class Manifest(Cloneable):
    """
    The Manifest class is used to maintain Manifest entry names and their
    associated Attributes. There are main Manifest Attributes as well as
    per-entry Attributes. For information on the Manifest format, please
    see the
    <a href="/../specs/jar/jar.html">
    Manifest format specification</a>.

    Author(s)
    - David Connelly

    See
    - Attributes

    Since
    - 1.2
    """

    def __init__(self):
        """
        Constructs a new, empty Manifest.
        """
        ...


    def __init__(self, is: "InputStream"):
        """
        Constructs a new Manifest from the specified input stream.

        Arguments
        - is: the input stream containing manifest data

        Raises
        - IOException: if an I/O error has occurred
        """
        ...


    def __init__(self, man: "Manifest"):
        """
        Constructs a new Manifest that is a copy of the specified Manifest.

        Arguments
        - man: the Manifest to copy
        """
        ...


    def getMainAttributes(self) -> "Attributes":
        """
        Returns the main Attributes for the Manifest.

        Returns
        - the main Attributes for the Manifest
        """
        ...


    def getEntries(self) -> dict[str, "Attributes"]:
        """
        Returns a Map of the entries contained in this Manifest. Each entry
        is represented by a String name (key) and associated Attributes (value).
        The Map permits the `null` key, but no entry with a null key is
        created by .read, nor is such an entry written by using .write.

        Returns
        - a Map of the entries contained in this Manifest
        """
        ...


    def getAttributes(self, name: str) -> "Attributes":
        """
        Returns the Attributes for the specified entry name.
        This method is defined as:
        ```
             return (Attributes)getEntries().get(name)
        ```
        Though `null` is a valid `name`, when
        `getAttributes(null)` is invoked on a `Manifest`
        obtained from a jar file, `null` will be returned.  While jar
        files themselves do not allow `null`-named attributes, it is
        possible to invoke .getEntries on a `Manifest`, and
        on that result, invoke `put` with a null key and an
        arbitrary value.  Subsequent invocations of
        `getAttributes(null)` will return the just-`put`
        value.
        
        Note that this method does not return the manifest's main attributes;
        see .getMainAttributes.

        Arguments
        - name: entry name

        Returns
        - the Attributes for the specified entry name
        """
        ...


    def clear(self) -> None:
        """
        Clears the main Attributes as well as the entries in this Manifest.
        """
        ...


    def write(self, out: "OutputStream") -> None:
        """
        Writes the Manifest to the specified OutputStream.
        Attributes.Name.MANIFEST_VERSION must be set in
        MainAttributes prior to invoking this method.

        Arguments
        - out: the output stream

        Raises
        - IOException: if an I/O error has occurred

        See
        - .getMainAttributes
        """
        ...


    def read(self, is: "InputStream") -> None:
        """
        Reads the Manifest from the specified InputStream. The entry
        names and attributes read will be merged in with the current
        manifest entries.

        Arguments
        - is: the input stream

        Raises
        - IOException: if an I/O error has occurred
        """
        ...


    def equals(self, o: "Object") -> bool:
        """
        Returns True if the specified Object is also a Manifest and has
        the same main Attributes and entries.

        Arguments
        - o: the object to be compared

        Returns
        - True if the specified Object is also a Manifest and has
        the same main Attributes and entries
        """
        ...


    def hashCode(self) -> int:
        """
        Returns the hash code for this Manifest.
        """
        ...


    def clone(self) -> "Object":
        """
        Returns a shallow copy of this Manifest.  The shallow copy is
        implemented as follows:
        ```
            public Object clone() { return new Manifest(this); }
        ```

        Returns
        - a shallow copy of this Manifest
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.EntityCategory

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.enchantments import Enchantment
from org.bukkit.entity import *
from org.bukkit.potion import PotionEffectType
from typing import Any, Callable, Iterable, Tuple


class EntityCategory(Enum):
    """
    A classification of entities which may behave differently than others or be
    affected uniquely by enchantments and potion effects among other things.
    """

    NONE = 0
    """
    Any uncategorized entity. No additional effects are applied to these
    entities relating to a categorization.
    """
    UNDEAD = 1
    """
    Undead creatures. These creatures:
    
      - Are damaged by potions of healing.
      - Are healed by potions of harming.
      - Are immune to drowning and poison.
      - Are subject to burning in daylight (though not all).
      - Sink in water (except Drowned, Phantom Phantoms
      and Wither Withers).
      - Take additional damage from Enchantment.SMITE.
      - Are ignored by Wither Withers.
    """
    ARTHROPOD = 2
    """
    Entities of the arthropod family. These creatures:
    
      - Take additional damage and receive PotionEffectType.SLOWNESS
      from Enchantment.BANE_OF_ARTHROPODS.
      - Are immune to PotionEffectType.POISON if they are spiders.
    """
    ILLAGER = 3
    """
    Entities that participate in raids. These creatures:
    
      - Are immune to damage from EvokerFangs.
      - Are ignored by Vindicator vindicators named "Johnny".
      - Are hostile to Villager villagers,
      WanderingTrader wandering traders, IronGolem iron golems
      and Player players.
    """
    WATER = 4
    """
    Entities that reside primarily underwater (excluding Drowned).
    These creatures:
    
      - Take additional damage from Enchantment.IMPALING.
      - Are immune to drowning (excluding Dolphin dolphins).
      - Take suffocation damage when out of water for extended periods of
      time (excluding Guardian guardians and Turtle turtles).
      - Are capable of swimming in water rather than floating or sinking.
    """



"""
Python module generated from Java source file java.lang.reflect.AnnotatedElement

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.reflect import *
from java.util import Arrays
from java.util import Objects
from java.util.function import Function
from java.util.stream import Collectors
from sun.reflect.annotation import AnnotationSupport
from sun.reflect.annotation import AnnotationType
from typing import Any, Callable, Iterable, Tuple


class AnnotatedElement:
    """
    Represents an annotated construct of the program currently running
    in this VM.
    
    A construct is either an element or a type. Annotations on an
    element are on a *declaration*, whereas annotations on a
    type are on a specific *use* of a type name.
    
    As defined by <cite>The Java Language Specification</cite>
    section 9.7.4, an annotation on an element is a
    *declaration annotation* and an annotation on a type is a
    *type annotation*.
    
    Note that any annotations returned by methods on the AnnotatedType AnnotatedType interface and its subinterfaces are
    type annotations as the entity being potentially annotated is a
    type. Annotations returned by methods outside of the `AnnotatedType` hierarchy are declaration annotations.
    
    This interface allows annotations to be read reflectively.  All
    annotations returned by methods in this interface are immutable and
    serializable. The arrays returned by methods of this interface may
    be modified by callers without affecting the arrays returned to
    other callers.
    
    The .getAnnotationsByType(Class) and .getDeclaredAnnotationsByType(Class) methods support multiple
    annotations of the same type on an element. If the argument to
    either method is a repeatable annotation type (JLS 9.6),
    then the method will "look through" a container annotation (JLS
    9.7), if present, and return any annotations inside the
    container. Container annotations may be generated at compile-time
    to wrap multiple annotations of the argument type.
    
    The terms *directly present*, *indirectly present*,
    *present*, and *associated* are used throughout this
    interface to describe precisely which annotations are returned by
    methods:
    
    
    
    -  An annotation *A* is *directly present* on an
    element *E* if *E* has a `RuntimeVisibleAnnotations` or `RuntimeVisibleParameterAnnotations` or `RuntimeVisibleTypeAnnotations` attribute, and the attribute
    contains *A*.
    
    - An annotation *A* is *indirectly present* on an
    element *E* if *E* has a `RuntimeVisibleAnnotations` or
    `RuntimeVisibleParameterAnnotations` or `RuntimeVisibleTypeAnnotations`
    attribute, and *A* 's type is repeatable, and the attribute contains
    exactly one annotation whose value element contains *A* and whose
    type is the containing annotation type of *A* 's type.
    
    - An annotation *A* is *present* on an element *E* if either:
    
    
    
    - *A* is directly present on *E*; or
    
    - No annotation of *A* 's type is directly present on
    *E*, and *E* is a class, and *A* 's type is
    inheritable, and *A* is present on the superclass of *E*.
    
    
    
    - An annotation *A* is *associated* with an element *E*
    if either:
    
    
    
    - *A* is directly or indirectly present on *E*; or
    
    - No annotation of *A* 's type is directly or indirectly
    present on *E*, and *E* is a class, and *A*'s type
    is inheritable, and *A* is associated with the superclass of
    *E*.
    
    
    
    
    
    The table below summarizes which kind of annotation presence
    different methods in this interface examine.
    
    <table class="plain">
    <caption>Overview of kind of presence detected by different AnnotatedElement methods</caption>
    <thead>
    <tr><th colspan=2 scope="col">Method</th>
        <th colspan=4 scope="col">Kind of Presence</th>
    <tr><th scope="col">Return Type</th>
        <th scope="col">Signature</th>
        <th scope="col">Directly Present</th>
        <th scope="col">Indirectly Present</th>
        <th scope="col">Present</th>
        <th scope="col">Associated</th>
    </thead>
    <tbody>
    <tr><td style="text-align:right">`T`</td>
    <th scope="row" style="font-weight:normal; text-align:left">.getAnnotation(Class) getAnnotation(Class&lt;T&gt;)
    <td></td><td></td><td style="text-align:center">X</td><td></td>
    </tr>
    <tr><td style="text-align:right">`Annotation[]`</td>
    <th scope="row" style="font-weight:normal; text-align:left">.getAnnotations getAnnotations()
    <td></td><td></td><td style="text-align:center">X</td><td></td>
    </tr>
    <tr><td style="text-align:right">`T[]`</td>
    <th scope="row" style="font-weight:normal; text-align:left">.getAnnotationsByType(Class) getAnnotationsByType(Class&lt;T&gt;)
    <td></td><td></td><td></td><td style="text-align:center">X</td>
    </tr>
    <tr><td style="text-align:right">`T`</td>
    <th scope="row" style="font-weight:normal; text-align:left">.getDeclaredAnnotation(Class) getDeclaredAnnotation(Class&lt;T&gt;)
    <td style="text-align:center">X</td><td></td><td></td><td></td>
    </tr>
    <tr><td style="text-align:right">`Annotation[]`</td>
    <th scope="row" style="font-weight:normal; text-align:left">.getDeclaredAnnotations getDeclaredAnnotations()
    <td style="text-align:center">X</td><td></td><td></td><td></td>
    </tr>
    <tr><td style="text-align:right">`T[]`</td>
    <th scope="row" style="font-weight:normal; text-align:left">.getDeclaredAnnotationsByType(Class) getDeclaredAnnotationsByType(Class&lt;T&gt;)
    <td style="text-align:center">X</td><td style="text-align:center">X</td><td></td><td></td>
    </tr>
    </tbody>
    </table>
    
    For an invocation of `get[Declared]AnnotationsByType(Class <T>)`,
    the order of annotations which are directly or indirectly
    present on an element *E* is computed as if indirectly present
    annotations on *E* are directly present on *E* in place
    of their container annotation, in the order in which they appear in
    the value element of the container annotation.
    
    There are several compatibility concerns to keep in mind if an
    annotation type *T* is originally *not* repeatable and
    later modified to be repeatable.
    
    The containing annotation type for *T* is *TC*.
    
    
    
    - Modifying *T* to be repeatable is source and binary
    compatible with existing uses of *T* and with existing uses
    of *TC*.
    
    That is, for source compatibility, source code with annotations of
    type *T* or of type *TC* will still compile. For binary
    compatibility, class files with annotations of type *T* or of
    type *TC* (or with other kinds of uses of type *T* or of
    type *TC*) will link against the modified version of *T*
    if they linked against the earlier version.
    
    (An annotation type *TC* may informally serve as an acting
    containing annotation type before *T* is modified to be
    formally repeatable. Alternatively, when *T* is made
    repeatable, *TC* can be introduced as a new type.)
    
    - If an annotation type *TC* is present on an element, and
    *T* is modified to be repeatable with *TC* as its
    containing annotation type then:
    
    
    
    - The change to *T* is behaviorally compatible with respect
    to the `get[Declared]Annotation(Class<T>)` (called with an
    argument of *T* or *TC*) and `get[Declared]Annotations()` methods because the results of the
    methods will not change due to *TC* becoming the containing
    annotation type for *T*.
    
    - The change to *T* changes the results of the `get[Declared]AnnotationsByType(Class<T>)` methods called with an
    argument of *T*, because those methods will now recognize an
    annotation of type *TC* as a container annotation for *T*
    and will "look through" it to expose annotations of type *T*.
    
    
    
    - If an annotation of type *T* is present on an
    element and *T* is made repeatable and more annotations of
    type *T* are added to the element:
    
    
    
    -  The addition of the annotations of type *T* is both
    source compatible and binary compatible.
    
    - The addition of the annotations of type *T* changes the results
    of the `get[Declared]Annotation(Class<T>)` methods and `get[Declared]Annotations()` methods, because those methods will now
    only see a container annotation on the element and not see an
    annotation of type *T*.
    
    - The addition of the annotations of type *T* changes the
    results of the `get[Declared]AnnotationsByType(Class<T>)`
    methods, because their results will expose the additional
    annotations of type *T* whereas previously they exposed only a
    single annotation of type *T*.
    
    
    
    
    
    If an annotation returned by a method in this interface contains
    (directly or indirectly) a Class-valued member referring to
    a class that is not accessible in this VM, attempting to read the class
    by calling the relevant Class-returning method on the returned annotation
    will result in a TypeNotPresentException.
    
    Similarly, attempting to read an enum-valued member will result in
    a EnumConstantNotPresentException if the enum constant in the
    annotation is no longer present in the enum class.
    
    If an annotation type *T* is (meta-)annotated with an
    `@Repeatable` annotation whose value element indicates a type
    *TC*, but *TC* does not declare a `value()` method
    with a return type of *T*`[]`, then an exception of type
    java.lang.annotation.AnnotationFormatError is thrown.
    
    Finally, attempting to read a member whose definition has evolved
    incompatibly will result in a java.lang.annotation.AnnotationTypeMismatchException or an
    java.lang.annotation.IncompleteAnnotationException.

    Author(s)
    - Josh Bloch

    See
    - java.lang.annotation.IncompleteAnnotationException

    Since
    - 1.5
    """

    def isAnnotationPresent(self, annotationClass: type["Annotation"]) -> bool:
        """
        Returns True if an annotation for the specified type
        is *present* on this element, else False.  This method
        is designed primarily for convenient access to marker annotations.
        
        The truth value returned by this method is equivalent to:
        `getAnnotation(annotationClass) != null`

        Arguments
        - annotationClass: the Class object corresponding to the
               annotation type

        Returns
        - True if an annotation for the specified annotation
            type is present on this element, else False

        Raises
        - NullPointerException: if the given annotation class is null

        Since
        - 1.5

        Unknown Tags
        - The default implementation returns `getAnnotation(annotationClass) != null`.
        """
        ...


    def getAnnotation(self, annotationClass: type["T"]) -> "T":
        """
        Returns this element's annotation for the specified type if
        such an annotation is *present*, else null.
        
        Type `<T>`: the type of the annotation to query for and return if present

        Arguments
        - annotationClass: the Class object corresponding to the
               annotation type

        Returns
        - this element's annotation for the specified annotation type if
            present on this element, else null

        Raises
        - NullPointerException: if the given annotation class is null

        Since
        - 1.5
        """
        ...


    def getAnnotations(self) -> list["Annotation"]:
        """
        Returns annotations that are *present* on this element.
        
        If there are no annotations *present* on this element, the return
        value is an array of length 0.
        
        The caller of this method is free to modify the returned array; it will
        have no effect on the arrays returned to other callers.

        Returns
        - annotations present on this element

        Since
        - 1.5
        """
        ...


    def getAnnotationsByType(self, annotationClass: type["T"]) -> list["T"]:
        """
        Returns annotations that are *associated* with this element.
        
        If there are no annotations *associated* with this element, the return
        value is an array of length 0.
        
        The difference between this method and .getAnnotation(Class)
        is that this method detects if its argument is a *repeatable
        annotation type* (JLS 9.6), and if so, attempts to find one or
        more annotations of that type by "looking through" a container
        annotation.
        
        The caller of this method is free to modify the returned array; it will
        have no effect on the arrays returned to other callers.
        
        Type `<T>`: the type of the annotation to query for and return if present

        Arguments
        - annotationClass: the Class object corresponding to the
               annotation type

        Returns
        - all this element's annotations for the specified annotation type if
            associated with this element, else an array of length zero

        Raises
        - NullPointerException: if the given annotation class is null

        Since
        - 1.8

        Unknown Tags
        - The default implementation first calls .getDeclaredAnnotationsByType(Class) passing `annotationClass` as the argument. If the returned array has
        length greater than zero, the array is returned. If the returned
        array is zero-length and this `AnnotatedElement` is a
        class and the argument type is an inheritable annotation type,
        and the superclass of this `AnnotatedElement` is non-null,
        then the returned result is the result of calling .getAnnotationsByType(Class) on the superclass with `annotationClass` as the argument. Otherwise, a zero-length
        array is returned.
        """
        ...


    def getDeclaredAnnotation(self, annotationClass: type["T"]) -> "T":
        """
        Returns this element's annotation for the specified type if
        such an annotation is *directly present*, else null.
        
        This method ignores inherited annotations. (Returns null if no
        annotations are directly present on this element.)
        
        Type `<T>`: the type of the annotation to query for and return if directly present

        Arguments
        - annotationClass: the Class object corresponding to the
               annotation type

        Returns
        - this element's annotation for the specified annotation type if
            directly present on this element, else null

        Raises
        - NullPointerException: if the given annotation class is null

        Since
        - 1.8

        Unknown Tags
        - The default implementation first performs a null check
        and then loops over the results of .getDeclaredAnnotations returning the first annotation whose
        annotation type matches the argument type.
        """
        ...


    def getDeclaredAnnotationsByType(self, annotationClass: type["T"]) -> list["T"]:
        """
        Returns this element's annotation(s) for the specified type if
        such annotations are either *directly present* or
        *indirectly present*. This method ignores inherited
        annotations.
        
        If there are no specified annotations directly or indirectly
        present on this element, the return value is an array of length
        0.
        
        The difference between this method and .getDeclaredAnnotation(Class) is that this method detects if its
        argument is a *repeatable annotation type* (JLS 9.6), and if so,
        attempts to find one or more annotations of that type by "looking
        through" a container annotation if one is present.
        
        The caller of this method is free to modify the returned array; it will
        have no effect on the arrays returned to other callers.
        
        Type `<T>`: the type of the annotation to query for and return
        if directly or indirectly present

        Arguments
        - annotationClass: the Class object corresponding to the
               annotation type

        Returns
        - all this element's annotations for the specified annotation type if
            directly or indirectly present on this element, else an array of length zero

        Raises
        - NullPointerException: if the given annotation class is null

        Since
        - 1.8

        Unknown Tags
        - The default implementation may call .getDeclaredAnnotation(Class) one or more times to find a
        directly present annotation and, if the annotation type is
        repeatable, to find a container annotation. If annotations of
        the annotation type `annotationClass` are found to be both
        directly and indirectly present, then .getDeclaredAnnotations() will get called to determine the
        order of the elements in the returned array.
        
        Alternatively, the default implementation may call .getDeclaredAnnotations() a single time and the returned array
        examined for both directly and indirectly present
        annotations. The results of calling .getDeclaredAnnotations() are assumed to be consistent with the
        results of calling .getDeclaredAnnotation(Class).
        """
        ...


    def getDeclaredAnnotations(self) -> list["Annotation"]:
        """
        Returns annotations that are *directly present* on this element.
        This method ignores inherited annotations.
        
        If there are no annotations *directly present* on this element,
        the return value is an array of length 0.
        
        The caller of this method is free to modify the returned array; it will
        have no effect on the arrays returned to other callers.

        Returns
        - annotations directly present on this element

        Since
        - 1.5
        """
        ...



"""
Python module generated from Java source file com.google.common.reflect.TypeToInstanceMap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.reflect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import DoNotMock
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import NonNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class TypeToInstanceMap(Map):
    """
    A map, each entry of which maps a TypeToken to an instance of that type. In addition to
    implementing `Map`, the additional type-safe operations .putInstance and .getInstance are available.
    
    Generally, implementations don't support .put and .putAll because there is no
    way to check an object at runtime to be an instance of a TypeToken. Instead, caller
    should use the type safe .putInstance.
    
    Also, if caller suppresses unchecked warnings and passes in an `Iterable<String>` for
    type `Iterable<Integer>`, the map won't be able to detect and throw type error.
    
    Like any other `Map<Class, Object>`, this map may contain entries for primitive types,
    and a primitive type and its corresponding wrapper type may map to different values.
    
    Type `<B>`: the common supertype that all entries must share; often this is simply Object

    Author(s)
    - Ben Yu

    Since
    - 13.0
    """

    def getInstance(self, type: type["T"]) -> "T":
        """
        Returns the value the specified class is mapped to, or `null` if no entry for this class
        is present. This will only return a value that was bound to this specific class, not a value
        that may have been bound to a subtype.
        
        `getInstance(Foo.class)` is equivalent to `getInstance(TypeToken.of(Foo.class))`.
        """
        ...


    def getInstance(self, type: "TypeToken"["T"]) -> "T":
        """
        Returns the value the specified type is mapped to, or `null` if no entry for this type is
        present. This will only return a value that was bound to this specific type, not a value that
        may have been bound to a subtype.
        """
        ...


    def putInstance(self, type: type["T"], value: "T") -> "T":
        """
        Maps the specified class to the specified value. Does *not* associate this value with any
        of the class's supertypes.
        
        `putInstance(Foo.class, foo)` is equivalent to `putInstance(TypeToken.of(Foo.class), foo)`.

        Returns
        - the value previously associated with this class (possibly `null`), or `null` if there was no previous entry.
        """
        ...


    def putInstance(self, type: "TypeToken"["T"], value: "T") -> "T":
        """
        Maps the specified type to the specified value. Does *not* associate this value with any
        of the type's supertypes.

        Returns
        - the value previously associated with this type (possibly `null`), or `null`
            if there was no previous entry.
        """
        ...



"""
Python module generated from Java source file org.bukkit.util.Consumer

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.util import *
from typing import Any, Callable, Iterable, Tuple


class Consumer(Consumer):

    def accept(self, t: "T") -> None:
        """
        Performs this operation on the given argument.

        Arguments
        - t: the input argument
        """
        ...



from .OldEnum import OldEnum
from .CachedServerIcon import CachedServerIcon
from .StructureSearchResult import StructureSearchResult
from .ChatPaginator import ChatPaginator
from .BlockVector import BlockVector
from .VoxelShape import VoxelShape
from .Transformation import Transformation
from .Consumer import Consumer
from .BoundingBox import BoundingBox
from .StringUtil import StringUtil
from .FileUtil import FileUtil
from .EulerAngle import EulerAngle
from .RayTraceResult import RayTraceResult
from .EntityTransformer import EntityTransformer
from .BlockIterator import BlockIterator
from .NumberConversions import NumberConversions
from .BiomeSearchResult import BiomeSearchResult
from .BlockTransformer import BlockTransformer
from .Vector import Vector


"""
Python module generated from Java source file org.bukkit.entity.Spellcaster

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class Spellcaster(Illager):
    """
    Represents a spell casting "Illager".
    """

    def getSpell(self) -> "Spell":
        """
        Gets the Spell the entity is currently using.

        Returns
        - the current spell
        """
        ...


    def setSpell(self, spell: "Spell") -> None:
        """
        Sets the Spell the entity is currently using.

        Arguments
        - spell: the spell the entity should be using
        """
        ...


    class Spell(Enum):
        """
        Represents the current spell the entity is using.
        """

        NONE = 0
        """
        No spell is being used..
        """
        SUMMON_VEX = 1
        """
        The spell that summons Vexes.
        """
        FANGS = 2
        """
        The spell that summons Fangs.
        """
        WOLOLO = 3
        """
        The "wololo" spell.
        """
        DISAPPEAR = 4
        """
        The spell that makes the casting entity invisible.
        """
        BLINDNESS = 5
        """
        The spell that makes the target blind.
        """



"""
Python module generated from Java source file java.lang.reflect.InvocationHandler

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.invoke import MethodHandle
from java.lang.reflect import *
from java.util import Objects
from jdk.internal.reflect import CallerSensitive
from jdk.internal.reflect import Reflection
from typing import Any, Callable, Iterable, Tuple


class InvocationHandler:
    """
    `InvocationHandler` is the interface implemented by
    the *invocation handler* of a proxy instance.
    
    Each proxy instance has an associated invocation handler.
    When a method is invoked on a proxy instance, the method
    invocation is encoded and dispatched to the `invoke`
    method of its invocation handler.

    Author(s)
    - Peter Jones

    See
    - Proxy

    Since
    - 1.3
    """

    def invoke(self, proxy: "Object", method: "Method", args: list["Object"]) -> "Object":
        """
        Processes a method invocation on a proxy instance and returns
        the result.  This method will be invoked on an invocation handler
        when a method is invoked on a proxy instance that it is
        associated with.

        Arguments
        - proxy: the proxy instance that the method was invoked on
        - method: the `Method` instance corresponding to
        the interface method invoked on the proxy instance.  The declaring
        class of the `Method` object will be the interface that
        the method was declared in, which may be a superinterface of the
        proxy interface that the proxy class inherits the method through.
        - args: an array of objects containing the values of the
        arguments passed in the method invocation on the proxy instance,
        or `null` if interface method takes no arguments.
        Arguments of primitive types are wrapped in instances of the
        appropriate primitive wrapper class, such as
        `java.lang.Integer` or `java.lang.Boolean`.

        Returns
        - the value to return from the method invocation on the
        proxy instance.  If the declared return type of the interface
        method is a primitive type, then the value returned by
        this method must be an instance of the corresponding primitive
        wrapper class; otherwise, it must be a type assignable to the
        declared return type.  If the value returned by this method is
        `null` and the interface method's return type is
        primitive, then a `NullPointerException` will be
        thrown by the method invocation on the proxy instance.  If the
        value returned by this method is otherwise not compatible with
        the interface method's declared return type as described above,
        a `ClassCastException` will be thrown by the method
        invocation on the proxy instance.

        Raises
        - Throwable: the exception to throw from the method
        invocation on the proxy instance.  The exception's type must be
        assignable either to any of the exception types declared in the
        `throws` clause of the interface method or to the
        unchecked exception types `java.lang.RuntimeException`
        or `java.lang.Error`.  If a checked exception is
        thrown by this method that is not assignable to any of the
        exception types declared in the `throws` clause of
        the interface method, then an
        UndeclaredThrowableException containing the
        exception that was thrown by this method will be thrown by the
        method invocation on the proxy instance.

        See
        - UndeclaredThrowableException
        """
        ...


    @staticmethod
    def invokeDefault(proxy: "Object", method: "Method", *args: Tuple["Object", ...]) -> "Object":
        """
        Invokes the specified default method on the given `proxy` instance with
        the given parameters.  The given `method` must be a default method
        declared in a proxy interface of the `proxy`'s class or inherited
        from its superinterface directly or indirectly.
        
        Invoking this method behaves as if `invokespecial` instruction executed
        from the proxy class, targeting the default method in a proxy interface.
        This is equivalent to the invocation:
        `X.super.m(A* a)` where `X` is a proxy interface and the call to
        `X.super::m(A*)` is resolved to the given `method`.
        
        Examples: interface `A` and `B` both declare a default
        implementation of method `m`. Interface `C` extends `A`
        and inherits the default method `m` from its superinterface `A`.
        
        <blockquote>````interface A {
            default T m(A a) { return t1;`
        }
        interface B {
            default T m(A a) { return t2; }
        }
        interface C extends A {}
        }```</blockquote>
        
        The following creates a proxy instance that implements `A`
        and invokes the default method `A::m`.
        
        <blockquote>````Object proxy = Proxy.newProxyInstance(loader, new Class<?>[] { A.class`,
                (o, m, params) -> {
                    if (m.isDefault()) {
                        // if it's a default method, invoke it
                        return InvocationHandler.invokeDefault(o, m, params);
                    }
                });
        }```</blockquote>
        
        If a proxy instance implements both `A` and `B`, both
        of which provides the default implementation of method `m`,
        the invocation handler can dispatch the method invocation to
        `A::m` or `B::m` via the `invokeDefault` method.
        For example, the following code delegates the method invocation
        to `B::m`.
        
        <blockquote>````Object proxy = Proxy.newProxyInstance(loader, new Class<?>[] { A.class, B.class`,
                (o, m, params) -> {
                    if (m.getName().equals("m")) {
                        // invoke B::m instead of A::m
                        Method bMethod = B.class.getMethod(m.getName(), m.getParameterTypes());
                        return InvocationHandler.invokeDefault(o, bMethod, params);
                    }
                });
        }```</blockquote>
        
        If a proxy instance implements `C` that inherits the default
        method `m` from its superinterface `A`, then
        the interface method invocation on `"m"` is dispatched to
        the invocation handler's .invoke(Object, Method, Object[]) invoke
        method with the `Method` object argument representing the
        default method `A::m`.
        
        <blockquote>````Object proxy = Proxy.newProxyInstance(loader, new Class<?>[] { C.class`,
               (o, m, params) -> {
                    if (m.isDefault()) {
                        // behaves as if calling C.super.m(params)
                        return InvocationHandler.invokeDefault(o, m, params);
                    }
               });
        }```</blockquote>
        
        The invocation of method `"m"` on this `proxy` will behave
        as if `C.super::m` is called and that is resolved to invoking
        `A::m`.
        
        Adding a default method, or changing a method from abstract to default
        may cause an exception if an existing code attempts to call `invokeDefault`
        to invoke a default method.
        
        For example, if `C` is modified to implement a default method
        `m`:
        
        <blockquote>````interface C extends A {
            default T m(A a) { return t3;`
        }
        }```</blockquote>
        
        The code above that creates proxy instance `proxy` with
        the modified `C` will run with no exception and it will result in
        calling `C::m` instead of `A::m`.
        
        The following is another example that creates a proxy instance of `C`
        and the invocation handler calls the `invokeDefault` method
        to invoke `A::m`:
        
        <blockquote>````C c = (C) Proxy.newProxyInstance(loader, new Class<?>[] { C.class`,
                (o, m, params) -> {
                    if (m.getName().equals("m")) {
                        // IllegalArgumentException thrown as `A::m` is not a method
                        // inherited from its proxy interface C
                        Method aMethod = A.class.getMethod(m.getName(), m.getParameterTypes());
                        return InvocationHandler.invokeDefault(o, aMethod params);
                    }
                });
        c.m(...);
        }```</blockquote>
        
        The above code runs successfully with the old version of `C` and
        `A::m` is invoked.  When running with the new version of `C`,
        the above code will fail with `IllegalArgumentException` because
        `C` overrides the implementation of the same method and
        `A::m` is not accessible by a proxy instance.

        Arguments
        - proxy: the `Proxy` instance on which the default method to be invoked
        - method: the `Method` instance corresponding to a default method
                       declared in a proxy interface of the proxy class or inherited
                       from its superinterface directly or indirectly
        - args: the parameters used for the method invocation; can be `null`
                       if the number of formal parameters required by the method is zero.

        Returns
        - the value returned from the method invocation

        Raises
        - IllegalArgumentException: if any of the following conditions is `True`:
                
                - `proxy` is not Proxy.isProxyClass(Class)
                    a proxy instance; or
                - the given `method` is not a default method declared
                    in a proxy interface of the proxy class and not inherited from
                    any of its superinterfaces; or
                - the given `method` is overridden directly or indirectly by
                    the proxy interfaces and the method reference to the named
                    method never resolves to the given `method`; or
                - the length of the given `args` array does not match the
                    number of parameters of the method to be invoked; or
                - any of the `args` elements fails the unboxing
                    conversion if the corresponding method parameter type is
                    a primitive type; or if, after possible unboxing, any of the
                    `args` elements cannot be assigned to the corresponding
                    method parameter type.
                
        - IllegalAccessException: if the declaring class of the specified
                default method is inaccessible to the caller class
        - NullPointerException: if `proxy` or `method` is `null`
        - Throwable: anything thrown by the default method

        Since
        - 16

        Unknown Tags
        - The `proxy` parameter is of type `Object` rather than `Proxy`
        to make it easy for InvocationHandler.invoke(Object, Method, Object[])
        InvocationHandler::invoke implementation to call directly without the need
        of casting.
        - 5.4.3. Method Resolution
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.ComputationException

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class ComputationException(RuntimeException):
    """
    Wraps an exception that occurred during a computation.

    Author(s)
    - Bob Lee

    Since
    - 2.0

    Deprecated
    - This exception is no longer thrown by `com.google.common`. Previously, it was
        thrown by MapMaker computing maps. When support for computing maps was removed from
        `MapMaker`, it was added to `CacheBuilder`, which throws `ExecutionException`, `UncheckedExecutionException`, and `ExecutionError`. Any
        code that is still catching `ComputationException` may need to be updated to catch some
        of those types instead. (Note that this type, though deprecated, is not planned to be removed
        from Guava.)
    """

    def __init__(self, cause: "Throwable"):
        """
        Creates a new instance with the given cause.
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.block.BlockFertilizeEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Block
from org.bukkit.block import BlockState
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from org.bukkit.event.world import StructureGrowEvent
from typing import Any, Callable, Iterable, Tuple


class BlockFertilizeEvent(BlockEvent, Cancellable):
    """
    Called with the block changes resulting from a player fertilizing a given
    block with bonemeal. Will be called after the applicable
    StructureGrowEvent.
    """

    def __init__(self, theBlock: "Block", player: "Player", blocks: list["BlockState"]):
        ...


    def getPlayer(self) -> "Player":
        """
        Gets the player that triggered the fertilization.

        Returns
        - triggering player, or null if not applicable
        """
        ...


    def getBlocks(self) -> list["BlockState"]:
        """
        Gets a list of all blocks changed by the fertilization.

        Returns
        - list of all changed blocks
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancelled: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerInputEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Input
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from typing import Any, Callable, Iterable, Tuple


class PlayerInputEvent(PlayerEvent):
    """
    This event is called when a player sends updated input to the server.

    See
    - Player.getCurrentInput()
    """

    def __init__(self, player: "Player", input: "Input"):
        ...


    def getInput(self) -> "Input":
        """
        Gets the new input received from this player.

        Returns
        - the new input
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.nodes.Node

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml.comments import CommentLine
from org.yaml.snakeyaml.error import Mark
from org.yaml.snakeyaml.nodes import *
from typing import Any, Callable, Iterable, Tuple


class Node:
    """
    Base class for all nodes.
    
    The nodes form the node-graph described in the <a href="http://yaml.org/spec/1.1/">YAML
    Specification</a>.
    
    
    While loading, the node graph is usually created by the
    org.yaml.snakeyaml.composer.Composer, and later transformed into application specific
    Java classes by the classes from the org.yaml.snakeyaml.constructor package.
    """

    def __init__(self, tag: "Tag", startMark: "Mark", endMark: "Mark"):
        ...


    def getTag(self) -> "Tag":
        """
        Tag of this node.
        
        Every node has a tag assigned. The tag is either local or global.

        Returns
        - Tag of this node.
        """
        ...


    def getEndMark(self) -> "Mark":
        ...


    def getNodeId(self) -> "NodeId":
        """
        For error reporting.

        Returns
        - scalar, sequence, mapping

        See
        - "class variable 'id' in PyYAML"
        """
        ...


    def getStartMark(self) -> "Mark":
        ...


    def setTag(self, tag: "Tag") -> None:
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Node is only equal to itself
        """
        ...


    def getType(self) -> type["Object"]:
        ...


    def setType(self, type: type["Object"]) -> None:
        ...


    def setTwoStepsConstruction(self, twoStepsConstruction: bool) -> None:
        ...


    def isTwoStepsConstruction(self) -> bool:
        """
        Indicates if this node must be constructed in two steps.
        
        Two-step construction is required whenever a node is a child (direct or indirect) of it self.
        That is, if a recursive structure is build using anchors and aliases.
        
        
        Set by org.yaml.snakeyaml.composer.Composer, used during the construction process.
        
        
        Only relevant during loading.

        Returns
        - `True` if the node is self referenced.
        """
        ...


    def hashCode(self) -> int:
        ...


    def useClassConstructor(self) -> bool:
        ...


    def setUseClassConstructor(self, useClassConstructor: "Boolean") -> None:
        ...


    def getAnchor(self) -> str:
        ...


    def setAnchor(self, anchor: str) -> None:
        ...


    def getInLineComments(self) -> list["CommentLine"]:
        """
        The ordered list of in-line comments. The first of which appears at the end of the line
        respresent by this node. The rest are in the following lines, indented per the Spec to indicate
        they are continuation of the inline comment.

        Returns
        - the comment line list.
        """
        ...


    def setInLineComments(self, inLineComments: list["CommentLine"]) -> None:
        ...


    def getBlockComments(self) -> list["CommentLine"]:
        """
        The ordered list of blank lines and block comments (full line) that appear before this node.

        Returns
        - the comment line list.
        """
        ...


    def setBlockComments(self, blockComments: list["CommentLine"]) -> None:
        ...


    def getEndComments(self) -> list["CommentLine"]:
        """
        The ordered list of blank lines and block comments (full line) that appear AFTER this node.
        
        NOTE: these comment should occur only in the last node in a document, when walking the node
        tree "in order"

        Returns
        - the comment line list.
        """
        ...


    def setEndComments(self, endComments: list["CommentLine"]) -> None:
        ...



"""
Python module generated from Java source file org.bukkit.event.entity.AreaEffectCloudApplyEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import AreaEffectCloud
from org.bukkit.entity import LivingEntity
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class AreaEffectCloudApplyEvent(EntityEvent, Cancellable):
    """
    Called when a lingering potion applies it's effects. Happens
    once every 5 ticks
    """

    def __init__(self, entity: "AreaEffectCloud", affectedEntities: list["LivingEntity"]):
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getEntity(self) -> "AreaEffectCloud":
        ...


    def getAffectedEntities(self) -> list["LivingEntity"]:
        """
        Retrieves a mutable list of the effected entities
        
        It is important to note that not every entity in this list
        is guaranteed to be effected.  The cloud may die during the
        application of its effects due to the depletion of AreaEffectCloud.getDurationOnUse()
        or AreaEffectCloud.getRadiusOnUse()

        Returns
        - the affected entity list
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file net.md_5.bungee.api.chat.TranslatableComponent

Java source file obtained from artifact bungeecord-chat version 1.20-R0.2-20240119.213604-65

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.regex import Matcher
from java.util.regex import Pattern
from net.md_5.bungee.api.chat import *
from net.md_5.bungee.chat import TranslationRegistry
from typing import Any, Callable, Iterable, Tuple


class TranslatableComponent(BaseComponent):

    def __init__(self, original: "TranslatableComponent"):
        """
        Creates a translatable component from the original to clone it.

        Arguments
        - original: the original for the new translatable component.
        """
        ...


    def __init__(self, translate: str, *with: Tuple["Object", ...]):
        """
        Creates a translatable component with the passed substitutions

        Arguments
        - translate: the translation key
        - with: the java.lang.Strings and
        net.md_5.bungee.api.chat.BaseComponents to use into the
        translation

        See
        - .setWith(java.util.List)
        """
        ...


    def __init__(self, translatable: "TranslationProvider", *with: Tuple["Object", ...]):
        """
        Creates a translatable component with the passed substitutions

        Arguments
        - translatable: the translatable object
        - with: the java.lang.Strings and
        net.md_5.bungee.api.chat.BaseComponents to use into the
        translation

        See
        - .setWith(java.util.List)
        """
        ...


    def duplicate(self) -> "TranslatableComponent":
        """
        Creates a duplicate of this TranslatableComponent.

        Returns
        - the duplicate of this TranslatableComponent.
        """
        ...


    def setWith(self, components: list["BaseComponent"]) -> None:
        """
        Sets the translation substitutions to be used in this component. Removes
        any previously set substitutions

        Arguments
        - components: the components to substitute
        """
        ...


    def addWith(self, text: str) -> None:
        """
        Adds a text substitution to the component. The text will inherit this
        component's formatting

        Arguments
        - text: the text to substitute
        """
        ...


    def addWith(self, component: "BaseComponent") -> None:
        """
        Adds a component substitution to the component. The text will inherit
        this component's formatting

        Arguments
        - component: the component to substitute
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.SpawnCategory

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class SpawnCategory(Enum):
    """
    Represents groups of entities with shared spawn behaviors and mob caps.

    See
    - <a href="https://minecraft.wiki/w/Spawn.Java_Edition_mob_cap">Minecraft Wiki</a>
    """

    MONSTER = 0
    """
    Entities related to Monsters, eg: Witch, Zombie, Creeper, etc.
    """
    ANIMAL = 1
    """
    Entities related to Animals, eg: Strider, Cow, Turtle, etc.
    """
    WATER_ANIMAL = 2
    """
    Entities related to Water Animals, eg: Squid or Dolphin.
    """
    WATER_AMBIENT = 3
    """
    Entities related to Water Ambient, eg: Cod, PufferFish, Tropical Fish,
    Salmon, etc.
    """
    WATER_UNDERGROUND_CREATURE = 4
    """
    Entities related to Water Underground, eg: Glow Squid.
    """
    AMBIENT = 5
    """
    Entities related to Ambient, eg: Bat.
    """
    AXOLOTL = 6
    """
    All the Axolotl are represented by this Category.
    """
    MISC = 7
    """
    Entities not related to a mob, eg: Player, ArmorStand, Boat, etc.
    """



"""
Python module generated from Java source file java.util.concurrent.locks.ReadWriteLock

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.concurrent.locks import *
from typing import Any, Callable, Iterable, Tuple


class ReadWriteLock:
    """
    A `ReadWriteLock` maintains a pair of associated Lock locks, one for read-only operations and one for writing.
    The .readLock read lock may be held simultaneously
    by multiple reader threads, so long as there are no writers.
    The .writeLock write lock is exclusive.
    
    All `ReadWriteLock` implementations must guarantee that
    the memory synchronization effects of `writeLock` operations
    (as specified in the Lock interface) also hold with respect
    to the associated `readLock`. That is, a thread successfully
    acquiring the read lock will see all updates made upon previous
    release of the write lock.
    
    A read-write lock allows for a greater level of concurrency in
    accessing shared data than that permitted by a mutual exclusion lock.
    It exploits the fact that while only a single thread at a time (a
    *writer* thread) can modify the shared data, in many cases any
    number of threads can concurrently read the data (hence *reader*
    threads).
    In theory, the increase in concurrency permitted by the use of a read-write
    lock will lead to performance improvements over the use of a mutual
    exclusion lock. In practice this increase in concurrency will only be fully
    realized on a multi-processor, and then only if the access patterns for
    the shared data are suitable.
    
    Whether or not a read-write lock will improve performance over the use
    of a mutual exclusion lock depends on the frequency that the data is
    read compared to being modified, the duration of the read and write
    operations, and the contention for the data - that is, the number of
    threads that will try to read or write the data at the same time.
    For example, a collection that is initially populated with data and
    thereafter infrequently modified, while being frequently searched
    (such as a directory of some kind) is an ideal candidate for the use of
    a read-write lock. However, if updates become frequent then the data
    spends most of its time being exclusively locked and there is little, if any
    increase in concurrency. Further, if the read operations are too short
    the overhead of the read-write lock implementation (which is inherently
    more complex than a mutual exclusion lock) can dominate the execution
    cost, particularly as many read-write lock implementations still serialize
    all threads through a small section of code. Ultimately, only profiling
    and measurement will establish whether the use of a read-write lock is
    suitable for your application.
    
    Although the basic operation of a read-write lock is straight-forward,
    there are many policy decisions that an implementation must make, which
    may affect the effectiveness of the read-write lock in a given application.
    Examples of these policies include:
    
    - Determining whether to grant the read lock or the write lock, when
    both readers and writers are waiting, at the time that a writer releases
    the write lock. Writer preference is common, as writes are expected to be
    short and infrequent. Reader preference is less common as it can lead to
    lengthy delays for a write if the readers are frequent and long-lived as
    expected. Fair, or &quot;in-order&quot; implementations are also possible.
    
    - Determining whether readers that request the read lock while a
    reader is active and a writer is waiting, are granted the read lock.
    Preference to the reader can delay the writer indefinitely, while
    preference to the writer can reduce the potential for concurrency.
    
    - Determining whether the locks are reentrant: can a thread with the
    write lock reacquire it? Can it acquire a read lock while holding the
    write lock? Is the read lock itself reentrant?
    
    - Can the write lock be downgraded to a read lock without allowing
    an intervening writer? Can a read lock be upgraded to a write lock,
    in preference to other waiting readers or writers?
    
    
    You should consider all of these things when evaluating the suitability
    of a given implementation for your application.

    Author(s)
    - Doug Lea

    See
    - ReentrantLock

    Since
    - 1.5
    """

    def readLock(self) -> "Lock":
        """
        Returns the lock used for reading.

        Returns
        - the lock used for reading
        """
        ...


    def writeLock(self) -> "Lock":
        """
        Returns the lock used for writing.

        Returns
        - the lock used for writing
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.util.EnumUtils

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml.util import *
from typing import Any, Callable, Iterable, Tuple


class EnumUtils:
    """
    Helper
    """

    @staticmethod
    def findEnumInsensitiveCase(enumType: type["T"], name: str) -> "T":
        """
        Looks for an enumeration constant that matches the string without being case sensitive
        
        Type `<T>`: - the enum type whose constant is to be returned

        Arguments
        - enumType: - the Class object of the enum type from which to return a constant
        - name: - the name of the constant to return

        Returns
        - the enum constant of the specified enum type with the specified name, insensitive to
                case

        Raises
        - IllegalArgumentException: – if the specified enum type has no constant with the
                specified name, insensitive case
        """
        ...



"""
Python module generated from Java source file java.util.ArrayDeque

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import Serializable
from java.util import *
from java.util.function import Consumer
from java.util.function import Predicate
from jdk.internal.access import SharedSecrets
from typing import Any, Callable, Iterable, Tuple


class ArrayDeque(AbstractCollection, Deque, Cloneable, Serializable):
    """
    Resizable-array implementation of the Deque interface.  Array
    deques have no capacity restrictions; they grow as necessary to support
    usage.  They are not thread-safe; in the absence of external
    synchronization, they do not support concurrent access by multiple threads.
    Null elements are prohibited.  This class is likely to be faster than
    Stack when used as a stack, and faster than LinkedList
    when used as a queue.
    
    Most `ArrayDeque` operations run in amortized constant time.
    Exceptions include
    .remove(Object) remove,
    .removeFirstOccurrence removeFirstOccurrence,
    .removeLastOccurrence removeLastOccurrence,
    .contains contains,
    .iterator iterator.remove(),
    and the bulk operations, all of which run in linear time.
    
    The iterators returned by this class's .iterator() iterator
    method are *fail-fast*: If the deque is modified at any time after
    the iterator is created, in any way except through the iterator's own
    `remove` method, the iterator will generally throw a ConcurrentModificationException.  Thus, in the face of concurrent
    modification, the iterator fails quickly and cleanly, rather than risking
    arbitrary, non-deterministic behavior at an undetermined time in the
    future.
    
    Note that the fail-fast behavior of an iterator cannot be guaranteed
    as it is, generally speaking, impossible to make any hard guarantees in the
    presence of unsynchronized concurrent modification.  Fail-fast iterators
    throw `ConcurrentModificationException` on a best-effort basis.
    Therefore, it would be wrong to write a program that depended on this
    exception for its correctness: *the fail-fast behavior of iterators
    should be used only to detect bugs.*
    
    This class and its iterator implement all of the
    *optional* methods of the Collection and Iterator interfaces.
    
    This class is a member of the
    <a href="/java.base/java/util/package-summary.html#CollectionsFramework">
    Java Collections Framework</a>.
    
    Type `<E>`: the type of elements held in this deque

    Author(s)
    - Josh Bloch and Doug Lea

    Since
    - 1.6
    """

    def __init__(self):
        """
        Constructs an empty array deque with an initial capacity
        sufficient to hold 16 elements.
        """
        ...


    def __init__(self, numElements: int):
        """
        Constructs an empty array deque with an initial capacity
        sufficient to hold the specified number of elements.

        Arguments
        - numElements: lower bound on initial capacity of the deque
        """
        ...


    def __init__(self, c: Iterable["E"]):
        """
        Constructs a deque containing the elements of the specified
        collection, in the order they are returned by the collection's
        iterator.  (The first element returned by the collection's
        iterator becomes the first element, or *front* of the
        deque.)

        Arguments
        - c: the collection whose elements are to be placed into the deque

        Raises
        - NullPointerException: if the specified collection is null
        """
        ...


    def addFirst(self, e: "E") -> None:
        """
        Inserts the specified element at the front of this deque.

        Arguments
        - e: the element to add

        Raises
        - NullPointerException: if the specified element is null
        """
        ...


    def addLast(self, e: "E") -> None:
        """
        Inserts the specified element at the end of this deque.
        
        This method is equivalent to .add.

        Arguments
        - e: the element to add

        Raises
        - NullPointerException: if the specified element is null
        """
        ...


    def addAll(self, c: Iterable["E"]) -> bool:
        """
        Adds all of the elements in the specified collection at the end
        of this deque, as if by calling .addLast on each one,
        in the order that they are returned by the collection's iterator.

        Arguments
        - c: the elements to be inserted into this deque

        Returns
        - `True` if this deque changed as a result of the call

        Raises
        - NullPointerException: if the specified collection or any
                of its elements are null
        """
        ...


    def offerFirst(self, e: "E") -> bool:
        """
        Inserts the specified element at the front of this deque.

        Arguments
        - e: the element to add

        Returns
        - `True` (as specified by Deque.offerFirst)

        Raises
        - NullPointerException: if the specified element is null
        """
        ...


    def offerLast(self, e: "E") -> bool:
        """
        Inserts the specified element at the end of this deque.

        Arguments
        - e: the element to add

        Returns
        - `True` (as specified by Deque.offerLast)

        Raises
        - NullPointerException: if the specified element is null
        """
        ...


    def removeFirst(self) -> "E":
        """
        Raises
        - NoSuchElementException: 
        """
        ...


    def removeLast(self) -> "E":
        """
        Raises
        - NoSuchElementException: 
        """
        ...


    def pollFirst(self) -> "E":
        ...


    def pollLast(self) -> "E":
        ...


    def getFirst(self) -> "E":
        """
        Raises
        - NoSuchElementException: 
        """
        ...


    def getLast(self) -> "E":
        """
        Raises
        - NoSuchElementException: 
        """
        ...


    def peekFirst(self) -> "E":
        ...


    def peekLast(self) -> "E":
        ...


    def removeFirstOccurrence(self, o: "Object") -> bool:
        """
        Removes the first occurrence of the specified element in this
        deque (when traversing the deque from head to tail).
        If the deque does not contain the element, it is unchanged.
        More formally, removes the first element `e` such that
        `o.equals(e)` (if such an element exists).
        Returns `True` if this deque contained the specified element
        (or equivalently, if this deque changed as a result of the call).

        Arguments
        - o: element to be removed from this deque, if present

        Returns
        - `True` if the deque contained the specified element
        """
        ...


    def removeLastOccurrence(self, o: "Object") -> bool:
        """
        Removes the last occurrence of the specified element in this
        deque (when traversing the deque from head to tail).
        If the deque does not contain the element, it is unchanged.
        More formally, removes the last element `e` such that
        `o.equals(e)` (if such an element exists).
        Returns `True` if this deque contained the specified element
        (or equivalently, if this deque changed as a result of the call).

        Arguments
        - o: element to be removed from this deque, if present

        Returns
        - `True` if the deque contained the specified element
        """
        ...


    def add(self, e: "E") -> bool:
        """
        Inserts the specified element at the end of this deque.
        
        This method is equivalent to .addLast.

        Arguments
        - e: the element to add

        Returns
        - `True` (as specified by Collection.add)

        Raises
        - NullPointerException: if the specified element is null
        """
        ...


    def offer(self, e: "E") -> bool:
        """
        Inserts the specified element at the end of this deque.
        
        This method is equivalent to .offerLast.

        Arguments
        - e: the element to add

        Returns
        - `True` (as specified by Queue.offer)

        Raises
        - NullPointerException: if the specified element is null
        """
        ...


    def remove(self) -> "E":
        """
        Retrieves and removes the head of the queue represented by this deque.
        
        This method differs from .poll() poll() only in that it
        throws an exception if this deque is empty.
        
        This method is equivalent to .removeFirst.

        Returns
        - the head of the queue represented by this deque

        Raises
        - NoSuchElementException: 
        """
        ...


    def poll(self) -> "E":
        """
        Retrieves and removes the head of the queue represented by this deque
        (in other words, the first element of this deque), or returns
        `null` if this deque is empty.
        
        This method is equivalent to .pollFirst.

        Returns
        - the head of the queue represented by this deque, or
                `null` if this deque is empty
        """
        ...


    def element(self) -> "E":
        """
        Retrieves, but does not remove, the head of the queue represented by
        this deque.  This method differs from .peek peek only in
        that it throws an exception if this deque is empty.
        
        This method is equivalent to .getFirst.

        Returns
        - the head of the queue represented by this deque

        Raises
        - NoSuchElementException: 
        """
        ...


    def peek(self) -> "E":
        """
        Retrieves, but does not remove, the head of the queue represented by
        this deque, or returns `null` if this deque is empty.
        
        This method is equivalent to .peekFirst.

        Returns
        - the head of the queue represented by this deque, or
                `null` if this deque is empty
        """
        ...


    def push(self, e: "E") -> None:
        """
        Pushes an element onto the stack represented by this deque.  In other
        words, inserts the element at the front of this deque.
        
        This method is equivalent to .addFirst.

        Arguments
        - e: the element to push

        Raises
        - NullPointerException: if the specified element is null
        """
        ...


    def pop(self) -> "E":
        """
        Pops an element from the stack represented by this deque.  In other
        words, removes and returns the first element of this deque.
        
        This method is equivalent to .removeFirst().

        Returns
        - the element at the front of this deque (which is the top
                of the stack represented by this deque)

        Raises
        - NoSuchElementException: 
        """
        ...


    def size(self) -> int:
        """
        Returns the number of elements in this deque.

        Returns
        - the number of elements in this deque
        """
        ...


    def isEmpty(self) -> bool:
        """
        Returns `True` if this deque contains no elements.

        Returns
        - `True` if this deque contains no elements
        """
        ...


    def iterator(self) -> Iterator["E"]:
        """
        Returns an iterator over the elements in this deque.  The elements
        will be ordered from first (head) to last (tail).  This is the same
        order that elements would be dequeued (via successive calls to
        .remove or popped (via successive calls to .pop).

        Returns
        - an iterator over the elements in this deque
        """
        ...


    def descendingIterator(self) -> Iterator["E"]:
        ...


    def spliterator(self) -> "Spliterator"["E"]:
        """
        Creates a *<a href="Spliterator.html#binding">late-binding</a>*
        and *fail-fast* Spliterator over the elements in this
        deque.
        
        The `Spliterator` reports Spliterator.SIZED,
        Spliterator.SUBSIZED, Spliterator.ORDERED, and
        Spliterator.NONNULL.  Overriding implementations should document
        the reporting of additional characteristic values.

        Returns
        - a `Spliterator` over the elements in this deque

        Since
        - 1.8
        """
        ...


    def forEach(self, action: "Consumer"["E"]) -> None:
        """
        Raises
        - NullPointerException: 
        """
        ...


    def removeIf(self, filter: "Predicate"["E"]) -> bool:
        """
        Raises
        - NullPointerException: 
        """
        ...


    def removeAll(self, c: Iterable[Any]) -> bool:
        """
        Raises
        - NullPointerException: 
        """
        ...


    def retainAll(self, c: Iterable[Any]) -> bool:
        """
        Raises
        - NullPointerException: 
        """
        ...


    def contains(self, o: "Object") -> bool:
        """
        Returns `True` if this deque contains the specified element.
        More formally, returns `True` if and only if this deque contains
        at least one element `e` such that `o.equals(e)`.

        Arguments
        - o: object to be checked for containment in this deque

        Returns
        - `True` if this deque contains the specified element
        """
        ...


    def remove(self, o: "Object") -> bool:
        """
        Removes a single instance of the specified element from this deque.
        If the deque does not contain the element, it is unchanged.
        More formally, removes the first element `e` such that
        `o.equals(e)` (if such an element exists).
        Returns `True` if this deque contained the specified element
        (or equivalently, if this deque changed as a result of the call).
        
        This method is equivalent to .removeFirstOccurrence(Object).

        Arguments
        - o: element to be removed from this deque, if present

        Returns
        - `True` if this deque contained the specified element
        """
        ...


    def clear(self) -> None:
        """
        Removes all of the elements from this deque.
        The deque will be empty after this call returns.
        """
        ...


    def toArray(self) -> list["Object"]:
        """
        Returns an array containing all of the elements in this deque
        in proper sequence (from first to last element).
        
        The returned array will be "safe" in that no references to it are
        maintained by this deque.  (In other words, this method must allocate
        a new array).  The caller is thus free to modify the returned array.
        
        This method acts as bridge between array-based and collection-based
        APIs.

        Returns
        - an array containing all of the elements in this deque
        """
        ...


    def toArray(self, a: list["T"]) -> list["T"]:
        """
        Returns an array containing all of the elements in this deque in
        proper sequence (from first to last element); the runtime type of the
        returned array is that of the specified array.  If the deque fits in
        the specified array, it is returned therein.  Otherwise, a new array
        is allocated with the runtime type of the specified array and the
        size of this deque.
        
        If this deque fits in the specified array with room to spare
        (i.e., the array has more elements than this deque), the element in
        the array immediately following the end of the deque is set to
        `null`.
        
        Like the .toArray() method, this method acts as bridge between
        array-based and collection-based APIs.  Further, this method allows
        precise control over the runtime type of the output array, and may,
        under certain circumstances, be used to save allocation costs.
        
        Suppose `x` is a deque known to contain only strings.
        The following code can be used to dump the deque into a newly
        allocated array of `String`:
        
        ``` `String[] y = x.toArray(new String[0]);````
        
        Note that `toArray(new Object[0])` is identical in function to
        `toArray()`.

        Arguments
        - a: the array into which the elements of the deque are to
                 be stored, if it is big enough; otherwise, a new array of the
                 same runtime type is allocated for this purpose

        Returns
        - an array containing all of the elements in this deque

        Raises
        - ArrayStoreException: if the runtime type of the specified array
                is not a supertype of the runtime type of every element in
                this deque
        - NullPointerException: if the specified array is null
        """
        ...


    def clone(self) -> "ArrayDeque"["E"]:
        """
        Returns a copy of this deque.

        Returns
        - a copy of this deque
        """
        ...
